package com.zh.test_zabbix.handel;


import com.zh.test_zabbix.utils.BizException;
import com.zh.test_zabbix.utils.result.HttpStatusEnum;
import com.zh.test_zabbix.utils.result.Result;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.http.converter.HttpMessageNotReadableException;
import org.springframework.validation.BindException;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.RestControllerAdvice;

import javax.servlet.http.HttpServletRequest;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import javax.validation.ValidationException;
import java.util.stream.Collectors;

@RestControllerAdvice
@Slf4j
public class RestExceptionHandler {
    /**
     * 默认全局异常处理。
     * @param e the e
     * @return ResultData
     */

    @ExceptionHandler(value = {BindException.class, ValidationException.class, MethodArgumentNotValidException.class})
    public ResponseEntity<Result<String>> handleValidatedException(Exception e) {

        Result<String>  result = null;
        if (e instanceof  MethodArgumentNotValidException) {
            MethodArgumentNotValidException ex =(MethodArgumentNotValidException)  e;
            log.error("发生业务异常！原因是：{}",e.getMessage());
            result = Result.fail(HttpStatusEnum.BAD_REQUEST,
                ex.getBindingResult().getAllErrors().stream()
                .map(ObjectError::getDefaultMessage)
                .collect(Collectors.joining(";"))
            );
        } else  if (e instanceof ConstraintViolationException){
            ConstraintViolationException ex = (ConstraintViolationException) e;
            log.error("发生业务异常！原因是：{}",e.getMessage());
            result = Result.fail(HttpStatusEnum.BAD_REQUEST,
                ex.getConstraintViolations().stream()
                    .map(ConstraintViolation::getMessage)
                    .collect(Collectors.joining("; "))
            );
        }else  if (e instanceof BindException) {
            BindException  ex = (BindException ) e;
            log.error("发生业务异常！原因是：{}",e.getMessage());
            result = Result.fail(HttpStatusEnum.BAD_REQUEST,
                ex.getAllErrors().stream()
                    .map(ObjectError::getDefaultMessage)
                    .collect(Collectors.joining("; "))
            );
        }
        return new ResponseEntity<>(result,HttpStatus.BAD_REQUEST);
    }

    @ResponseStatus(HttpStatus.BAD_REQUEST)
    @ExceptionHandler(HttpMessageNotReadableException.class)
    @ResponseBody
    public Result<String> badRequestExceptionHandler(Exception e){
        log.error("发生业务异常！原因是：{}",e.getMessage());
        return Result.fail(HttpStatusEnum.BAD_REQUEST,"请求体内容有误:"+e.getMessage());
    }

    /**
     * 处理自定义的业务异常
     * @param req 请求
     * @param e 异常
     * @return
     */
    @ExceptionHandler(value = BizException.class)
    @ResponseBody
    public Result<String> bizExceptionHandler(HttpServletRequest req, BizException e){
        log.error("发生业务异常！原因是：{}",e.getMessage());
        return Result.fail(e.getCode(),e.getMessage());
    }
}
