package com.zh.test_zabbix.serviceImpl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zh.test_zabbix.config.Config;
import com.zh.test_zabbix.mapper.EventsMapper;
import com.zh.test_zabbix.model.Events;
import com.zh.test_zabbix.service.IEventsService;
import com.zh.test_zabbix.serviceImpl.util.ServiceUtil;
import com.zh.test_zabbix.utils.result.Result;
import lombok.Data;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author zh
 * @since 2023-11-14
 */
@Service
public class EventsServiceImpl extends ServiceImpl<EventsMapper, Events> implements IEventsService {

    @Resource
    private EventsMapper eventsMapper;

    @Data
    public static class ArgGet {
        Integer pageNum;
        Integer pageSize;

        ArrayList<Long> hostids;

        String time_from;
        String time_till;
    }

    @Override
    public Result get(ArgGet args) {
        QueryWrapper<Events> qw = new QueryWrapper<>();
        qw.eq("e.source", Config.Event.EVENT_SOURCE_TRIGGERS);
        qw.eq("e.object", Config.Event.EVENT_OBJECT_TRIGGER);
        qw.eq("e.value",Config.Trigger.TRIGGER_VALUE_TRUE);

        ArrayList<Long> hostids = args.hostids;
        if (null != hostids && hostids.size() > 0) {
            qw.in("i.hostid", hostids);
        }
        String time_from = args.time_from;
        if (null != time_from && "" != time_from) {
            qw.gt("e.clock", time_from);
        }
        String time_till = args.time_till;
        if (null != time_till && "" != time_till) {
            qw.lt("e.clock", time_till);
        }

        Integer count = eventsMapper.count(qw);
        ServiceUtil.Page page = ServiceUtil.init_page(args.pageNum, args.pageSize,count);
        qw.last("LIMIT " + (page.getNum() - 1) * page.getSize() + ", " + page.getSize());
        qw.orderByDesc("e.eventid");
        List<Map> result = eventsMapper.get(qw);

        return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT,result,count,page.getNum(),page.getSize()));
    }
}
