package com.zh.test_zabbix.serviceImpl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zh.test_zabbix.config.Config;
import com.zh.test_zabbix.mapper.GraphsMapper;
import com.zh.test_zabbix.mapper.HistoryMapper;
import com.zh.test_zabbix.mapper.res.ResGraphHostItems;
import com.zh.test_zabbix.mapper.res.ResHistory;
import com.zh.test_zabbix.model.Graphs;
import com.zh.test_zabbix.service.IGraphsService;
import com.zh.test_zabbix.serviceImpl.util.ServiceUtil;
import com.zh.test_zabbix.utils.result.HttpStatusEnum;
import com.zh.test_zabbix.utils.result.Result;
import lombok.Data;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author zh
 * @since 2023-11-13
 */
@Service
public class GraphsServiceImpl extends ServiceImpl<GraphsMapper, Graphs> implements IGraphsService {

    @Resource
    private GraphsMapper graphsMapper;

    @Resource
    private HistoryMapper historyMapper;

    @Data
    public static class ArgGet {
        Integer pageNum;
        Integer pageSize;
        ArrayList<Long> hostids;
        String name;

        Boolean selectParentItem;
        Boolean selectDiscovery;
        Boolean selectItems;
        Boolean selectHistory;

        String time_from;
        String time_till;
        String limit;

        Integer kind; // 1为模板，否则为主机
    }

    @Override
    public Result<JSONObject> get(ArgGet args) {

        // 主机和模板参数不同
        Boolean isTemplate = (null != args.kind && args.kind.equals(1)) ? true : false;

        QueryWrapper<Graphs> qw = new QueryWrapper<>();

        qw.in("g.flags", Arrays.asList(Config.host.ZBX_FLAG_DISCOVERY_NORMAL, Config.host.ZBX_FLAG_DISCOVERY_CREATED));
        if (isTemplate) {
            qw.eq("h.status", Config.host.HOST_STATUS_TEMPLATE);
        } else {
            qw.ne("h.status", Config.host.HOST_STATUS_TEMPLATE);
        }

        ArrayList<Long> arg_hostids = args.getHostids();
        if (null != arg_hostids && arg_hostids.size() > 0) {
            qw.in("h.hostid", arg_hostids);
        }

        String arg_name = args.getName();
        if (null != arg_name && "" != arg_name) {
            String arg_name_upper = arg_name.toUpperCase();
            qw.like("UPPER(g.name)", arg_name_upper);
        }

        Integer count = graphsMapper.count(qw);

        ArrayList<Long> ids = new ArrayList<>();
        ServiceUtil.Page page = ServiceUtil.init_page(args.pageNum, args.pageSize, count);
        qw.orderByAsc("UPPER(g.name)");
        qw.last("LIMIT " + (page.getNum() - 1) * page.getSize() + ", " + page.getSize());
        List<Graphs> graphsList = graphsMapper.ids(qw);
        for (Graphs g : graphsList) {
            ids.add(g.getGraphid());
        }
        if (ids.size() == 0) {
            return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT, new ArrayList<>()));
        }

        List<Graphs> result = graphsMapper.selectList(
                new QueryWrapper<Graphs>()
                        .in("graphid", ids)
                        .orderByAsc("UPPER(name)"));

        if (result.size() == 0) {
            Result.success(ServiceUtil.ResStruct(Config.MS_RESULT, result, count, page.getNum(), page.getSize()));
        }

        if (null != args.selectParentItem && args.selectParentItem) {
            ArrayList<Long> templateids = new ArrayList<>();
            for (Graphs r : result) {
                templateids.add(r.getTemplateid());
            }
            if (templateids.size() > 0) {
                ArrayList<ResGraphHostItems> graphTemplateHost = graphsMapper.graphHostItems(
                        new QueryWrapper<Graphs>()
                                .eq("h.status", Config.host.HOST_STATUS_TEMPLATE)
                                .in("h.flags", Arrays.asList(Config.host.ZBX_FLAG_DISCOVERY_NORMAL, Config.host.ZBX_FLAG_DISCOVERY_CREATED))
                                .in("gi.graphid", templateids));
                if (graphTemplateHost.size() > 0) {
                    for (Graphs r : result) {
                        if (null != r.getTemplateid() && r.getTemplateid() > 0) {
                            for (ResGraphHostItems gt : graphTemplateHost) {
                                if (Long.valueOf(gt.getGraphid()).equals(r.getTemplateid())) {
                                    r.setParent_hostid(Long.valueOf(gt.getHostid()));
                                    r.setParent_host_name(gt.getHost_name());
                                    r.setParent_itemid(Long.valueOf(gt.getItemid()));
                                    r.setParent_item_name(gt.getItem_name());
                                    break;
                                }
                            }
                        }
                    }
                }
            }
        }

        if (null != args.selectDiscovery && args.selectDiscovery) {
            ArrayList<Map> graphDiscoveries = graphsMapper.graphByItemDiscovery(
                    new QueryWrapper<Graphs>()
                            .eq("i.flags", Config.host.ZBX_FLAG_DISCOVERY_RULE)
                            .in("gd.graphid", ids));

            if (graphDiscoveries.size() > 0) {
                for (Graphs r : result) {
                    for (Map gd : graphDiscoveries) {
                        if (Long.valueOf(gd.get("graphid").toString()).equals(r.getGraphid())) {
                            r.setItem_discovery_itemid(Long.valueOf(gd.get("itemid").toString()));
                            r.setItem_discovery_name(gd.get("name").toString());
                            break;
                        }
                    }
                }
            }
        }

        if ((null != args.selectItems && args.selectItems) || (null != args.selectHistory && args.selectHistory)) {
            ArrayList<ResGraphHostItems> graphHostItems = graphsMapper.graphHostItems(
                    new QueryWrapper<Graphs>()
                            .in("h.status", Arrays.asList(Config.host.HOST_STATUS_MONITORED, Config.host.HOST_STATUS_NOT_MONITORED, Config.host.HOST_STATUS_TEMPLATE))
                            .in("h.flags", Arrays.asList(Config.host.ZBX_FLAG_DISCOVERY_NORMAL, Config.host.ZBX_FLAG_DISCOVERY_CREATED))
                            .in("i.flags", Arrays.asList(Config.host.ZBX_FLAG_DISCOVERY_NORMAL, Config.host.ZBX_FLAG_DISCOVERY_PROTOTYPE, Config.host.ZBX_FLAG_DISCOVERY_CREATED))
                            .in("gi.graphid", ids));

            for (Graphs r : result) {
                List<ResGraphHostItems> collect = graphHostItems.stream()
                        .filter(i -> i.getGraphid().equals(r.getGraphid()))
                        .collect(Collectors.toList());

                if (null != args.selectHistory && args.selectHistory) {
                    for (ResGraphHostItems gi : collect) {
                        QueryWrapper his_qw;
                        ArrayList<ResHistory> history = null;
                        his_qw = new QueryWrapper<>();
                        his_qw.eq("itemid", gi.getItemid());
                        his_qw.orderByDesc("clock", "ns");
//                        String time_from = args.time_from;
//                        if (null != time_from && "" != time_from) {
//                            qw.gt("clock", time_from);
//                        }
//                        String time_till = args.time_till;
//                        if (null != time_till && "" != time_till) {
//                            qw.lt("clock", time_till);
//                        }

                        if (Arrays.asList(Config.Graph.GRAPH_TYPE_NORMAL, Config.Graph.GRAPH_TYPE_STACKED).contains(r.getGraphtype())) {
                            Long time_from = System.currentTimeMillis() / 1000 - 60 * 60;
                            his_qw.gt("clock", time_from);
                        } else {
                            his_qw.last("LIMIT 1");
                        }

                        switch (gi.getValue_type()) {
                            case Config.item.ITEM_VALUE_TYPE_FLOAT:
                                history = historyMapper.history(his_qw);
                                break;
                            case Config.item.ITEM_VALUE_TYPE_STR:
                                history = historyMapper.history_str(his_qw);
                                break;
                            case Config.item.ITEM_VALUE_TYPE_LOG:
                                history = historyMapper.history_log(his_qw);
                                break;
                            case Config.item.ITEM_VALUE_TYPE_UINT64:
                                history = historyMapper.history_uint(his_qw);
                                break;
                            case Config.item.ITEM_VALUE_TYPE_TEXT:
                                history = historyMapper.history_text(his_qw);
                                break;
                        }
                        gi.setValue(history);
                    }
                }
                r.setItems(collect);
            }

        }

        return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT, result, count, page.getNum(), page.getSize()));
    }

    @Data
    public static class ArgDetail {
        Long graphid;
    }

    @Override
    public Result<JSONObject> detail(ArgDetail args) {
        Long graphid = args.graphid;
        QueryWrapper<Graphs> qw = new QueryWrapper<>();
        qw.eq("graphid", graphid);
        qw.in("flags", Arrays.asList(Config.host.ZBX_FLAG_DISCOVERY_NORMAL, Config.host.ZBX_FLAG_DISCOVERY_CREATED));
        Integer count = graphsMapper.selectCount(qw);
        if (!count.equals(1)) {
            return Result.fail(HttpStatusEnum.BAD_REQUEST, "无效id");
        }
        Graphs result = graphsMapper.selectOne(qw);

        ArrayList<Map> graphDiscoveries = graphsMapper.graphByItemDiscovery(
                new QueryWrapper<Graphs>()
                        .eq("i.flags", Config.host.ZBX_FLAG_DISCOVERY_RULE)
                        .eq("gd.graphid", graphid));
        if (graphDiscoveries.size() == 1) {
            result.setItem_discovery_itemid(Long.valueOf(graphDiscoveries.get(0).get("itemid").toString()));
            result.setItem_discovery_name(graphDiscoveries.get(0).get("name").toString());
        }

        Long templateid = result.getTemplateid();
        if (null != templateid && templateid > 0) {
            Integer count1 = graphsMapper.selectCount(
                    new QueryWrapper<Graphs>()
                            .in("flags", Arrays.asList(Config.host.ZBX_FLAG_DISCOVERY_NORMAL, Config.host.ZBX_FLAG_DISCOVERY_CREATED))
                            .eq("graphid", templateid));
            if (count1 == 1) {
                ArrayList<ResGraphHostItems> graphTemplateHost = graphsMapper.graphHostItems(
                        new QueryWrapper<Graphs>()
                                .eq("h.status", Config.host.HOST_STATUS_TEMPLATE)
                                .in("h.flags", Arrays.asList(Config.host.ZBX_FLAG_DISCOVERY_NORMAL, Config.host.ZBX_FLAG_DISCOVERY_CREATED))
                                .eq("gi.graphid", templateid));
                if (graphTemplateHost.size() > 0) {
                    result.setParent_hostid(Long.valueOf(graphTemplateHost.get(0).getHostid()));
                    result.setParent_host_name(graphTemplateHost.get(0).getHost_name());
                    result.setParent_itemid(Long.valueOf(graphTemplateHost.get(0).getItemid()));
                    result.setParent_item_name(graphTemplateHost.get(0).getItem_name());
                }
            }
        }

        ArrayList<ResGraphHostItems> graphHostItems = graphsMapper.graphHostItems(
                new QueryWrapper<Graphs>()
                        .in("h.status", Arrays.asList(Config.host.HOST_STATUS_MONITORED, Config.host.HOST_STATUS_NOT_MONITORED, Config.host.HOST_STATUS_TEMPLATE))
                        .in("h.flags", Arrays.asList(Config.host.ZBX_FLAG_DISCOVERY_NORMAL, Config.host.ZBX_FLAG_DISCOVERY_CREATED))
                        .in("i.flags", Arrays.asList(Config.host.ZBX_FLAG_DISCOVERY_NORMAL, Config.host.ZBX_FLAG_DISCOVERY_PROTOTYPE, Config.host.ZBX_FLAG_DISCOVERY_CREATED))
                        .eq("gi.graphid", graphid));
        result.setItems(graphHostItems);

        return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT, result));
    }


}
