package com.zh.test_zabbix.serviceImpl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zh.test_zabbix.config.Config;
import com.zh.test_zabbix.config.ItemEnum;
import com.zh.test_zabbix.mapper.HistoryMapper;
import com.zh.test_zabbix.mapper.HostItemsMapper;
import com.zh.test_zabbix.mapper.ItemsMapper;
import com.zh.test_zabbix.mapper.res.ResHistory;
import com.zh.test_zabbix.model.HostItems;
import com.zh.test_zabbix.model.Items;
import com.zh.test_zabbix.service.IHostItems;
import com.zh.test_zabbix.serviceImpl.util.ServiceUtil;
import com.zh.test_zabbix.utils.ZabbixApi;
import com.zh.test_zabbix.utils.result.HttpStatusEnum;
import com.zh.test_zabbix.utils.result.Result;
import lombok.Data;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.validation.constraints.NotNull;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;

@Service
public class HostItemsImpl extends ServiceImpl<HostItemsMapper, HostItems> implements IHostItems {



    @Resource
    private HistoryMapper historyMapper;

    @Resource
    private ItemsMapper itemsMapper;

    /* 初始化主机监控项 */
    public Boolean initHostItems(String[] hostids) {

        JSONObject params = new JSONObject();
        params.put("hostids", hostids);
        JSONObject res = ZabbixApi.post("item.get", Config.AUTH, params);

        // 分析主机下监控项信息
        ArrayList<HostItems> items = new ArrayList<>();
        try {
            JSONArray resitemResult = res.getJSONArray("result");
            for (Object i : resitemResult) {
                JSONObject obj = (JSONObject) i;
                String key_ = obj.getString("key_");
                Key_info key_info = split_key(key_);

                HostItems item = new HostItems()
                        .setItemid(obj.getIntValue("itemid"))
                        .setHostid(obj.getString("hostid"))
                        .setKey_(key_)
                        .setValueType(Integer.parseInt(obj.getString("value_type")))
                        .setName(obj.getString("name"))
                        .setKind(key_info.item_enum_id)
                        .setParam(key_info.param)
                        .setTarget(key_info.target);
                if (item.getKind() > 0) {
                    items.add(item);
                }
            }
        } catch (Exception e) {
            System.out.println("ERROR:方法item.get结果异常！");
//           log
            e.printStackTrace();
        }

        // 事务
        super.remove(new QueryWrapper<HostItems>().in("hostid", hostids));
        super.saveBatch(items);

        return true;
    }

    @Data
    static class Key_info {
        int item_enum_id = 0;
        String param;
        String target;
    }

    static Key_info split_key(String key) {
        Key_info key_ = new Key_info();
        int startIndex = key.indexOf("[");
        String commonKey = key;
        String param = null;
        if (startIndex > 0) {
            int endIndex = key.indexOf("]");
            param = key.substring(startIndex + 1, endIndex);
            commonKey = key.substring(0, startIndex);
        }
        ItemEnum itemEnum = ItemEnum.searchKeyId(commonKey);
        if (null != itemEnum) {
            Integer id = itemEnum.id();
            key_.item_enum_id = id;
            if (id > 1000 && id < 1999) {
                if (null != param) {
                    switch (id) {
                        case 1101:
                            key_.param = param.replace(",", "_");
                            break;
                        case 1102:
                        case 1301:
                            key_.param = param.replace(",", "");
                            break;
                        case 1201:
                            key_.param = param;
                            break;
                        case 1401:
                        case 1402:
                        case 1403:
                        case 1404:
                        case 1405:
                        case 1406:
                        case 1407:
                            key_.target = param;
                            break;
                        case 1501:
                        case 1502:
                            key_.target = param.substring(param.indexOf("\"") + 1, param.lastIndexOf("\""));
                            int paramIndex = param.indexOf(",");
                            if (paramIndex > 0) {
                                key_.param = param.substring(paramIndex + 1);
                            } else {
                                key_.param = "all";
                            }
                            break;
                        case 1503:
                            switch (param) {
                                case "\"/sys/class/net/eth0/operstate\"":
                                    key_.param = "operstate";
                                    key_.target = "eth0";
                                    break;
                                case "\"/sys/class/net/eth0/speed\"":
                                    key_.param = "speed";
                                    key_.target = "eth0";
                                    break;
                                case "\"/sys/class/net/eth0/type\"":
                                    key_.param = "type";
                                    key_.target = "eth0";
                                    break;
                            }
                            break;
                    }
                }
            }
        }
        return key_;
    }

    @Data
    public static class argInfo {
        @NotNull(message = "主机id不能为空")
        Long hostid;
        @NotNull(message = "类型不能为空")
        Integer type_;
        ArrayList<Integer> ids;    // itemEnum区间
    }

    public Result<JSONObject> info(argInfo arg) {
        ArrayList<HashMap<String, String>> l = new ArrayList<>();
        HashMap<String, Object> mapResp = new HashMap();

        // 获取监控项信息
        QueryWrapper<HostItems> qw = new QueryWrapper<>();
        qw.eq("hostid", arg.hostid);
        if (null != arg.ids && arg.ids.size() > 0) {
            qw.in("ROUND(kind / 100,0)", arg.ids);
        }
        List<HostItems> list = this.list(qw);


        // 获取监控项当前值
        ArrayList<Integer> itemids = new ArrayList<>();
        for (HostItems i : list) {
            itemids.add(i.getItemid());
        }
        JSONObject param = new JSONObject();
        param.put("itemids", itemids);
        ArrayList<String> outputArr = new ArrayList<>();
        outputArr.add("lastvalue");
        param.put("output", outputArr);
        JSONObject itemValResp = ZabbixApi.post("item.get", Config.AUTH, param);

        JSONArray itemHisRes = itemValResp.getJSONArray("result");
        HashMap<Integer, Object> itemVal = new HashMap<>();
        for (Object i : itemHisRes) {
            JSONObject obj = (JSONObject) i;
            itemVal.put(obj.getIntValue("itemid"), obj.get("lastvalue"));
        }

        HashMap<String, HashMap<String, Object>> mapList = new HashMap();
        for (HostItems i : list) {

            // 结构化监控项
            ItemEnum itemEnum = ItemEnum.valueOf(i.getKind());
            if (null != itemEnum) {
                String name = i.getParam();
                String target = i.getTarget();
                String word = itemEnum.word();
                // 监控项有多参数
                if (name != null) {
                    if (mapList.get(word) == null) {
                        mapList.put(word, new HashMap());
                    }
                    // 监控项有多采集对象
                    if (null != target) {
                        if (null == mapList.get(word).get(name)) {
                            mapList.get(word).put(name, new HashMap<>());
                        }
                        ((HashMap) mapList.get(word).get(name)).put(target, itemVal.get(i.getItemid()));
                    } else {
                        mapList.get(word).put(name, itemVal.get(i.getItemid()));
                    }
                } else {
                    // 监控项有多采集对象
                    if (null != target) {
                        if (null == mapList.get(word)) {
                            mapList.put(word, new HashMap<>());
                        }
                        mapList.get(word).put(target, itemVal.get(i.getItemid()));
                    } else {
                        mapResp.put(itemEnum.word(), itemVal.get(i.getItemid()));
                    }
                }
            }
        }

        mapResp.put("list", mapList);

        return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT, mapResp));
    }

    @Data
    public static class argHistory {
        @NotNull
        int hostid;
        @NotNull
        String key_;
    }

    public Result<JSONObject> hostItemHistory(argHistory args) {
        String  key_ = args.key_;
        QueryWrapper<Items> qw = new QueryWrapper<>();
        qw.inSql("hostid", "SELECT hostid from hosts where status <> " + Config.host.HOST_STATUS_TEMPLATE);
        qw.in("flags", Arrays.asList(Config.host.ZBX_FLAG_DISCOVERY_NORMAL, Config.host.ZBX_FLAG_DISCOVERY_CREATED));
        qw.eq("hostid",args.hostid);
        qw.eq("key_", key_);
        Integer item_count = itemsMapper.selectCount(qw);
        if (!item_count.equals(1)) {
            return Result.fail(HttpStatusEnum.BAD_REQUEST, "无效id");
        }

        Items items = itemsMapper.selectOne(qw);

        QueryWrapper his_qw = new QueryWrapper<>();
        his_qw.eq("itemid", items.getItemid());
        his_qw.orderByDesc("clock", "ns");
        his_qw.last("LIMIT 1000");

        ArrayList<ResHistory> history = null;
        switch (items.getValue_type()) {
            case Config.item.ITEM_VALUE_TYPE_FLOAT:
                history = historyMapper.history(his_qw);
                break;
            case Config.item.ITEM_VALUE_TYPE_STR:
                history = historyMapper.history_str(his_qw);
                break;
            case Config.item.ITEM_VALUE_TYPE_LOG:
                history = historyMapper.history_log(his_qw);
                break;
            case Config.item.ITEM_VALUE_TYPE_UINT64:
                history = historyMapper.history_uint(his_qw);
                break;
            case Config.item.ITEM_VALUE_TYPE_TEXT:
                history = historyMapper.history_text(his_qw);
                break;
        }
        return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT,history));
    }

}
