package com.zh.test_zabbix.serviceImpl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zh.test_zabbix.config.Config;
import com.zh.test_zabbix.mapper.HostsGroupsMapper;
import com.zh.test_zabbix.mapper.HostsMapper;
import com.zh.test_zabbix.mapper.HstgrpMapper;
import com.zh.test_zabbix.model.Hosts;
import com.zh.test_zabbix.model.HostsGroups;
import com.zh.test_zabbix.model.Hstgrp;
import com.zh.test_zabbix.service.IHostsGroupsService;
import com.zh.test_zabbix.serviceImpl.util.ServiceUtil;
import com.zh.test_zabbix.utils.ZabbixApi;
import com.zh.test_zabbix.utils.result.Result;
import lombok.Data;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.validation.constraints.NotNull;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author zh
 * @since 2023-11-14
 */
@Service
public class HostsGroupsServiceImpl extends ServiceImpl<HostsGroupsMapper, HostsGroups> implements IHostsGroupsService {

    @Resource
    private HostsGroupsMapper hostsGroupsMapper;

    @Resource
    private HstgrpMapper hstgrpMapper;

    @Resource
    private HostsMapper hostsMapper;


    @Data
    public static class ArgGet {
        Integer pageNum;
        Integer pageSize;
        Integer kind; // 1为模板，否则为主机
        String selectHosts;
    }

    @Override
    public Result<JSONObject> get(ArgGet args) {
        // 是否page格式
        Boolean isPage = true;
        if (null != args.pageSize && args.pageSize == 0) {
            isPage = false;
        }

        // 主机和模板参数不同
        Boolean isTemplate = (null != args.kind && args.kind.equals(1)) ? true : false;

        Integer host_group_type = isTemplate ? Config.host.HOST_GROUP_TYPE_TEMPLATE_GROUP : Config.host.HOST_GROUP_TYPE_HOST_GROUP;
        List<Integer> hosts_flags = Arrays.asList(Config.host.ZBX_FLAG_DISCOVERY_NORMAL, Config.host.ZBX_FLAG_DISCOVERY_CREATED);
        List<Integer> hosts_status = Arrays.asList(Config.host.HOST_STATUS_MONITORED, Config.host.HOST_STATUS_NOT_MONITORED);
        if (isTemplate) {
            hosts_status = Arrays.asList(Config.host.HOST_STATUS_TEMPLATE);
        }

        QueryWrapper<Hstgrp> qw = new QueryWrapper<>();
        qw.eq("type", host_group_type);
        Integer count = hstgrpMapper.selectCount(qw);
        if(count==0){
            return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT,new ArrayList<>()));
        }

        ServiceUtil.Page page = new ServiceUtil.Page();
        if (isPage) {
            page = ServiceUtil.init_page(args.pageNum, args.pageSize,count);
            qw.last("LIMIT " + (page.getNum() - 1) * page.getSize() + ", " + page.getSize());
        }
        qw.select("groupid", "name", "flags");
        qw.orderByAsc("name");
        List<Hstgrp> result = hstgrpMapper.selectList(qw);

        ArrayList<Long> groupids = new ArrayList<>();
        for (Hstgrp r : result) {
            groupids.add(r.getGroupid());
        }

        String selectHosts = args.selectHosts;
        if (null != selectHosts) {
            if (selectHosts.equals("count")) {
                QueryWrapper<HostsGroups> qw2 = new QueryWrapper<>();
                qw2.in(!isTemplate, " h.flags", hosts_flags);
                qw2.in(" h.status", hosts_status);
                qw2.in("hg.groupid", groupids);
                List<Map<String, Object>> hostsGroupsCount = hostsGroupsMapper.count(qw2);
                for (Hstgrp r : result) {
                    for (Map<String, Object> i : hostsGroupsCount) {
                        if (Long.valueOf(i.get("groupid").toString()).equals(r.getGroupid())) {
                            r.setHosts(i.get("rowscount"));
                            break;
                        }
                    }
                }
            } else if (selectHosts.equals("extend")) {
                QueryWrapper<HostsGroups> qw2 = new QueryWrapper<>();
                qw2.select("hostgroupid,groupid,hostid");
                qw2.in("groupid", groupids);
                List<HostsGroups> hostsGroups = hostsGroupsMapper.selectList(qw2);

                ArrayList<Long> hostids = new ArrayList<>();
                for (HostsGroups i : hostsGroups) {
                    hostids.add(i.getHostid());
                }
                if (hostids.size() > 0) {
                    QueryWrapper<Hosts> hosts_qw = new QueryWrapper<>();
                    hosts_qw.select("hostid,name,status");
                    hosts_qw.in(!isTemplate, " flags", hosts_flags);
                    hosts_qw.in(" status", hosts_status);
                    hosts_qw.in("hostid", hostids);
                    List<Hosts> hosts = hostsMapper.selectList(hosts_qw);
                    for (Hstgrp r : result) {
                        ArrayList<Object> resHosts = new ArrayList<>();
                        List<HostsGroups> collect = hostsGroups.stream()
                                .filter(i -> i.getGroupid().equals(r.getGroupid()))
                                .collect(Collectors.toList());
                        for (HostsGroups c : collect) {
                            for (Hosts i : hosts) {
                                if (i.getHostid().equals(c.getHostid())) {
                                    HashMap<String, Object> map = new HashMap<>();
                                    map.put("hostid", i.getHostid());
                                    map.put("name", i.getName());
                                    map.put("status", i.getStatus());
                                    resHosts.add(map);
                                }
                            }
                        }
                        r.setHosts(resHosts);
                    }
                }
            }
        }
        if (isPage) {
            return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT, result, count, page.getNum(), page.getSize()));
        } else {
            return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT, result));
        }
    }

    @Data
    public static class ArgCreate{
        @NotNull
        String name;
        Integer kind; // 1为模板，否则为主机
    }

    @Override
    public Result<JSONObject> create(ArgCreate args) {
        JSONObject params = new JSONObject();
        params.put("name", args.name);
        return  Result.success(ZabbixApi.post((null !=args.kind&& args.kind == 1)?"templategroup.create":"hostgroup.create", Config.AUTH, params));
    }

    @Data
    public static class ArgDelete{
        @NotNull
        ArrayList<Long> hostGroupIds;
        Integer kind; // 1为模板，否则为主机
    }

    @Override
    public Result<JSONObject> delete(ArgDelete args) {
        return  Result.success(ZabbixApi.post((null !=args.kind&& args.kind == 1)?"templategroup.delete":"hostgroup.delete", Config.AUTH, args.hostGroupIds));
    }

    @Data
    public static class ArgUpdate{
        @NotNull
        Long groupid;
        @NotNull
        String name;
        Integer kind; // 1为模板，否则为主机
    }

    @Override
    public Result<JSONObject> update(ArgUpdate args) {
        JSONObject params = new JSONObject();
        params.put("groupid", args.groupid);
        params.put("name", args.name);
        return  Result.success(ZabbixApi.post((null !=args.kind&& args.kind == 1)?"templategroup.update":"hostgroup.update", Config.AUTH, params));
    }

}
