package com.zh.test_zabbix.serviceImpl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zh.test_zabbix.config.Config;
import com.zh.test_zabbix.config.HostCategoryEnum;
import com.zh.test_zabbix.mapper.*;
import com.zh.test_zabbix.mapper.res.ResHostsTemplates;
import com.zh.test_zabbix.mapper.res.ResProblem;
import com.zh.test_zabbix.model.*;
import com.zh.test_zabbix.model.zabbixModel.host.HostInterface;
import com.zh.test_zabbix.script.AsyncTaskService;
import com.zh.test_zabbix.script.LinuxUtil;
import com.zh.test_zabbix.service.IHostsService;
import com.zh.test_zabbix.serviceImpl.util.ServiceUtil;
import com.zh.test_zabbix.utils.Common;
import com.zh.test_zabbix.utils.ZabbixApi;
import com.zh.test_zabbix.utils.result.HttpStatusEnum;
import com.zh.test_zabbix.utils.result.Result;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.validation.constraints.NotNull;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author zh
 * @since 2023-10-16
 */
@Service
public class HostsServiceImpl extends ServiceImpl<HostsMapper, Hosts> implements IHostsService {

    @Resource
    private HostsMapper hostsMapper;

    @Resource
    private HostRtdataMapper hostRtdataMapper;

    @Resource
    private InterfaceMapper interfaceMapper;

    @Resource
    private HostsGroupsMapper hostsGroupsMapper;

    @Resource
    private HstgrpMapper hstgrpMapper;

    @Resource
    private HostsTemplatesMapper hostsTemplatesMapper;

    @Resource
    private ItemsMapper itemsMapper;

    @Resource
    private TriggersMapper triggersMapper;

    @Resource
    private ProblemMapper problemMapper;

    @Resource
    private GraphsMapper graphsMapper;

    @Resource
    private HostTagMapper hostTagMapper;

    @Resource
    private HostmacroMapper hostmacroMapper;

    @Resource
    private ValuemapMapper valuemapMapper;

    @Resource
    private ValuemapMappingMapper valuemapMappingMapper;

    @Resource
    private MsHostCollectionMapper collectionMapper;

    @Resource
    private IdsMapper idsMapper;

    @Data
    public static class ArgGet {
        Integer pageNum;
        Integer pageSize;
        Long collectionid;
        ArrayList<Long> hostids;
        ArrayList<Long> groupids;
        ArrayList<Long> templateids;
        Integer kind; // 1为模板，否则为主机
        String name;
        String host;
        String dns;
        String ip;
        String port;
        Object selectGroups; // 返回 主机组 属性和主机所属的主机组数据。
        Object selectTemplates; // 返回 parentTemplates 属性和主机链接到的模板。支持 count.
        Object selectParentTemplates; // 返回 parentTemplates 属性和主机链接到的模板。支持 count.
        Object selectInterfaces; // 	返回 interfaces 属性和主机接口Supports count
        Object selectItems; // 返回 items 属性和主机项。支持 count.
        Object selectTriggers; // 返回 triggers 属性和主机触发器。支持 count.
        Object selectProblems;
        Object selectDiscoveries; // 	返回 发现 属性和主机低级发现规则。支持 count.
        //      Object selectDiscoveryRule; // 返回 发现规则 属性和创建主机的底层发现规则(来自VMware监控中的主机原型)。
        Object selectGraphs; // 	返回 图形 属性和主机图。支持 count.
        Object selectTags; // 返回 tags 属性和主机标记。
//        Object selectInheritedTags; // 	返回 inheritedTags 属性的标记，这些标记位于链接到主机的所有模板上
//        Object selectMacros; //	返回 macros 属性和主机宏。
//        Object selectValueMaps; // 	返回 valuemaps 属性和主机映射值。
//        Object selectHostDiscovery; // 返回 hostDiscovery 属性和主机发现对象数据。
////        主机发现对象将发现的主机链接到主机原型，或将主机原型链接到LLD规则，并具有以下属性:
////        host -(string)主机原型的主机;
////        hostid -(string)发现的主机或主机原型ID;
////        parent_hostid -(string)创建主机的主机原型ID;
////        parent_itemid -(string)发现主机的LLD规则ID;
////        lastcheck -(timestamp)最后发现主机的时间;
////        ts_delete -(timestamp)删除已发现主机的时间。
//        Object selectHttpTests; // 返回 httpTests 属性与主机web场景.支持 count
//        Object selectInventory; // 返回 inventory 属性和主机库存数据。
//        Object selectDashboards; // 返回 dashboards 属性.支持 count.

    }

    @Override
    public Result<JSONObject> get(ArgGet args) {

        // 主机和模板参数不同
        Boolean isTemplate = (null != args.kind && args.kind.equals(1)) ? true : false;

        QueryWrapper<Hosts> qw = new QueryWrapper<>();
        if (isTemplate) {
            qw.eq("status", Config.host.HOST_STATUS_TEMPLATE);
        } else {
            qw.in("status", Arrays.asList(Config.host.HOST_STATUS_MONITORED, Config.host.HOST_STATUS_NOT_MONITORED));
            qw.in("flags", Arrays.asList(Config.host.ZBX_FLAG_DISCOVERY_NORMAL, Config.host.ZBX_FLAG_DISCOVERY_CREATED));
        }

        ArrayList<Long> groupids = args.groupids;
        if (null != groupids && groupids.size() > 0) {
            qw.inSql("hostid", "SELECT hostid FROM hosts_groups WHERE groupid in (" + Common.getArrayList(groupids) + ")");
        }
        ArrayList<Long> templateids = args.templateids;
        if (null != templateids && templateids.size() > 0) {
            qw.inSql("hostid", "SELECT hostid FROM hosts_templates WHERE templateid in (" + Common.getArrayList(templateids) + ")");
        }
        String name = args.name;
        if (null != name && "" != name) {
            qw.like("name_upper", name.toUpperCase());
        }
        String arg_host = args.host;
        if (null != arg_host && "" != arg_host) {
            qw.like("upper(host)", arg_host.toUpperCase());
        }

        if (!isTemplate) {
            Long collectionid = args.collectionid;
            if (null != collectionid && collectionid > 0) {
                qw.eq("collectionid", collectionid);
            }
            String ip = args.ip;
            if (null != ip && "" != ip) {
                qw.inSql("hostid", "SELECT hostid from interface where upper(ip) like '%" + ip.toUpperCase() + "%'");
            }
            String dns = args.getDns();
            if (null != dns && "" != dns) {
                qw.inSql("hostid", "SELECT hostid from interface where upper(dns) like '%" + dns.toUpperCase() + "%'");
            }
            String port = args.port;
            if (null != port && "" != port) {
                qw.inSql("hostid", "SELECT hostid from interface where upper(port) like '%" + port.toUpperCase() + "%'");
            }
        }

        Integer count = hostsMapper.selectCount(qw);
        if (count == 0) {
            return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT, new ArrayList<>(), count));
        }

        ServiceUtil.Page page = ServiceUtil.init_page(args.pageNum, args.pageSize, count);
        qw.last("LIMIT " + (page.getNum() - 1) * page.getSize() + ", " + page.getSize());
        qw.orderByAsc("name_upper");
        List<Hosts> result = hostsMapper.selectList(qw);

        ArrayList<Long> hostids = new ArrayList<>();

        for (Hosts host : result) {
            hostids.add(host.getHostid());
        }

        if (null != args.selectGroups && "" != args.selectGroups) {
            List<HostsGroups> hostsGroups = hostsGroupsMapper.selectList(
                    new QueryWrapper<HostsGroups>()
                            .in("hostid", hostids));
            if (hostsGroups.size() > 0) {
                ArrayList<Long> hostsGroupids = new ArrayList<>();
                for (HostsGroups i : hostsGroups) {
                    hostsGroupids.add(i.getGroupid());
                }

                Integer hstgrp_type = isTemplate ? Config.host.HOST_GROUP_TYPE_TEMPLATE_GROUP : Config.host.HOST_GROUP_TYPE_HOST_GROUP;
                List<Hstgrp> hstgrps = hstgrpMapper.selectList(
                        new QueryWrapper<Hstgrp>()
                                .eq("type", hstgrp_type)
                                .in("groupid", hostsGroupids));
                for (Hosts r : result) {
                    ArrayList<Hstgrp> hs = new ArrayList<>();
                    for (HostsGroups hg : hostsGroups) {
                        if (hg.getHostid().equals(r.getHostid())) {
                            for (Hstgrp hstgrp : hstgrps) {
                                if (hstgrp.getGroupid().equals(hg.getGroupid())) {
                                    hs.add(hstgrp);
                                }
                            }
                        }
                    }
                    r.setGroups(hs);
                }
            }
        }

        if (!isTemplate) {
            ArrayList<Long> collectionids = new ArrayList<>();
            ArrayList<Long> categoryids = new ArrayList<>();
            for (Hosts host : result) {
                collectionids.add(host.getCollectionid());
                categoryids.add(host.getCategoryid());
            }

            List<HostRtdata> hostRtdatas = hostRtdataMapper.selectList(new QueryWrapper<HostRtdata>().select("active_available,hostid").in("hostid", hostids));
            for (Hosts r : result) {
                for (HostRtdata hostRtdata : hostRtdatas) {
                    if (hostRtdata.getHostid().equals(r.getHostid())) {
                        r.setAvailable(hostRtdata.getActive_available());
                        break;
                    }
                }
            }

            if (null != args.getSelectInterfaces()) {
                QueryWrapper<Interface> interface_qw = new QueryWrapper<>();
                interface_qw.in("hi.hostid", hostids);
                List<Map> hostsInterfaces = interfaceMapper.hostsInterface(interface_qw);
                for (Hosts r : result) {
                    List<Map> collect = hostsInterfaces.stream()
                            .filter(i -> Long.valueOf(i.get("hostid").toString()).equals(r.getHostid()))
                            .collect(Collectors.toList());
                    r.setInterfaces(collect);
                }
            }

            List<MsHostCollection> collections = collectionMapper.selectList(new QueryWrapper<MsHostCollection>().in("id", collectionids));
            for (Hosts r : result) {
                for (MsHostCollection coll : collections) {
                    if (coll.getId().equals(r.getCollectionid())) {
                        r.setCollection_name(coll.getName());
                        break;
                    }
                }
                HostCategoryEnum cate = HostCategoryEnum.valueOf(r.getCategoryid());
                if (null != cate) {
                    r.setCategory_name(cate.title());
                }
            }

            if (null != args.getSelectProblems()) {
                QueryWrapper<Problem> problem_qw = new QueryWrapper<>();
                problem_qw.eq("p.source", Config.Event.EVENT_SOURCE_TRIGGERS);
                problem_qw.eq("p.object", Config.Event.EVENT_OBJECT_TRIGGER);
                problem_qw.isNull("p.cause_eventid");
                problem_qw.isNull("p.r_eventid");
                problem_qw.in("h.hostid", hostids);
                List<ResProblem> problems = problemMapper.get(problem_qw);
                for (Hosts r : result) {
                    r.setProblems(0);
                    List<ResProblem> collect = problems.stream()
                            .filter(i -> i.getHostid().equals(r.getHostid()))
                            .collect(Collectors.toList());
                    r.setProblems(collect);
                }
            }

        } else {
            if (null != args.getSelectTemplates()) {
                QueryWrapper<HostsTemplates> hostsTemplates_qw = new QueryWrapper<>();
                hostsTemplates_qw.in("t.templateid", hostids);
                hostsTemplates_qw.eq("h.status", Config.host.HOST_STATUS_TEMPLATE);
                List<ResHostsTemplates> hostsTemplates = hostsTemplatesMapper.hostsTemplatesList(hostsTemplates_qw);
                for (Hosts r : result) {
                    List<ResHostsTemplates> collect = hostsTemplates.stream()
                            .filter(i -> i.getTemplateid().equals(r.getHostid()))
                            .collect(Collectors.toList());
                    r.setTemplates(collect);
                }
            }
        }

        if (null != args.getSelectParentTemplates()) {
            QueryWrapper<HostsTemplates> hostsTemplates_qw = new QueryWrapper<>();
            hostsTemplates_qw.in("t.hostid", hostids);
            hostsTemplates_qw.eq("h.status", Config.host.HOST_STATUS_TEMPLATE);
            List<ResHostsTemplates> hostsTemplates = hostsTemplatesMapper.hostsParentTemplatesList(hostsTemplates_qw);
            for (Hosts r : result) {
                List<ResHostsTemplates> collect = hostsTemplates.stream()
                        .filter(i -> i.getHostid().equals(r.getHostid()))
                        .collect(Collectors.toList());
                r.setParentTemplates(collect);
            }
        }

        if (null != args.getSelectItems()) {
            QueryWrapper<Items> items_qw = new QueryWrapper<>();
            items_qw.ne("type", Config.item.ITEM_TYPE_HTTPTEST);
            items_qw.in("flags", Arrays.asList(Config.host.ZBX_FLAG_DISCOVERY_NORMAL, Config.host.ZBX_FLAG_DISCOVERY_CREATED));
            items_qw.in("hostid", hostids);

            if (args.getSelectItems().equals("count")) {
                items_qw.groupBy("hostid");
                items_qw.select("COUNT(*) AS rowscount", "hostid");
            }
            List<Map<String, Object>> items = itemsMapper.selectMaps(items_qw);

            for (Hosts r : result) {
                r.setItems(0);
                for (Map<String, Object> i : items) {
                    if (Long.valueOf(i.get("hostid").toString()).equals(r.getHostid())) {
                        r.setItems(i.get("rowscount"));
                        break;
                    }
                }
            }
        }

        if (null != args.getSelectDiscoveries()) {
            QueryWrapper<Items> items_qw = new QueryWrapper<>();
            items_qw.in("flags", Arrays.asList(Config.host.ZBX_FLAG_DISCOVERY_RULE));
            items_qw.in("hostid", hostids);

            if (args.getSelectDiscoveries().equals("count")) {
                items_qw.groupBy("hostid");
                items_qw.select("COUNT(*) AS rowscount", "hostid");
            }
            List<Map<String, Object>> items = itemsMapper.selectMaps(items_qw);

            for (Hosts r : result) {
                r.setDiscoveries(0);
                for (Map<String, Object> i : items) {
                    if (Long.valueOf(i.get("hostid").toString()).equals(r.getHostid())) {
                        r.setDiscoveries(i.get("rowscount"));
                    }
                }
            }
        }

        if (null != args.getSelectTriggers()) {
            QueryWrapper<Triggers> trigger_qw = new QueryWrapper<>();
            trigger_qw.in("i.hostid", hostids);
            trigger_qw.in("t.flags", Arrays.asList(Config.host.ZBX_FLAG_DISCOVERY_NORMAL, Config.host.ZBX_FLAG_DISCOVERY_CREATED));
            List<Map> triggers = triggersMapper.countHostTrigger(trigger_qw);
            for (Hosts r : result) {
                r.setTriggers(0);
                for (Map<String, Object> i : triggers) {
                    if (Long.valueOf(i.get("hostid").toString()).equals(r.getHostid())) {
                        r.setTriggers(i.get("rowscount"));
                    }
                }
            }
        }

        if (null != args.selectGraphs) {
            QueryWrapper<Graphs> graphs_qw = new QueryWrapper<>();
            graphs_qw.in("i.hostid", hostids);
            graphs_qw.in("g.flags", Arrays.asList(Config.host.ZBX_FLAG_DISCOVERY_NORMAL, Config.host.ZBX_FLAG_DISCOVERY_CREATED));
            List<Map> graphs = graphsMapper.countHostGraphs(graphs_qw);
            for (Hosts r : result) {
                r.setGraphs(0);
                for (Map<String, Object> i : graphs) {
                    if (Long.valueOf(i.get("hostid").toString()).equals(r.getHostid())) {
                        r.setGraphs(i.get("rowscount"));
                    }
                }
            }
        }

        if (null != args.getSelectTags()) {
            QueryWrapper<HostTag> hostTag_qw = new QueryWrapper<>();
            hostTag_qw.in("hostid", hostids);
            List<HostTag> hostTags = hostTagMapper.selectList(hostTag_qw);
            for (Hosts r : result) {
                List<HostTag> collect = hostTags.stream()
                        .filter(i -> i.getHostid().equals(r.getHostid()))
                        .collect(Collectors.toList());
                r.setTags(collect);
            }
        }


        return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT, result, count, page.getNum(), page.getSize()));
    }

    @Data
    public static class ArgDetail {
        Long hostid;
        Integer kind; // 1为模板，否则为主机
        Boolean selectProblems;
    }

    @Override
    public Result<JSONObject> detail(ArgDetail args) {

        // 主机和模板参数不同
        Boolean isTemplate = (null != args.kind && args.kind.equals(1)) ? true : false;

        Long hostid = args.hostid;
        QueryWrapper<Hosts> qw = new QueryWrapper<>();
        if (isTemplate) {
            qw.eq("status", Config.host.HOST_STATUS_TEMPLATE);
        } else {
            qw.in("status", Arrays.asList(Config.host.HOST_STATUS_MONITORED, Config.host.HOST_STATUS_NOT_MONITORED));
        }
        qw.eq("hostid", hostid);
        Integer count = hostsMapper.selectCount(qw);
        if (!count.equals(1)) {
            return Result.fail(HttpStatusEnum.BAD_REQUEST, "无效id");
        }

        Hosts resHost = hostsMapper.selectOne(qw);

        if (!isTemplate) {

            List<Map> hostsInterfaces = interfaceMapper.hostsInterface(new QueryWrapper<Interface>().eq("hi.hostid", hostid));
            resHost.setInterfaces(hostsInterfaces);

            if (null != resHost.getCollectionid() && resHost.getCollectionid() > 0) {
                MsHostCollection collection = collectionMapper.selectOne(new QueryWrapper<MsHostCollection>().eq("id", resHost.getCollectionid()));
                resHost.setCollection_name(collection.getName());
            }
            HostCategoryEnum cate = HostCategoryEnum.valueOf(resHost.getCategoryid());
            if (null != cate) {
                resHost.setCategory_name(cate.title());
            }

            if (null != args.selectProblems && args.selectProblems) {
                QueryWrapper<Problem> problem_qw = new QueryWrapper<>();
                problem_qw.eq("p.source", Config.Event.EVENT_SOURCE_TRIGGERS);
                problem_qw.eq("p.object", Config.Event.EVENT_OBJECT_TRIGGER);
                problem_qw.isNull("p.cause_eventid");
                problem_qw.isNull("p.r_eventid");
                problem_qw.eq("h.hostid", hostid);
                List<ResProblem> problems = problemMapper.get(problem_qw);
                resHost.setProblems(problems);
            }
        }

        // 模板组列表
        List<HostsGroups> hostsGroups = hostsGroupsMapper.selectList(
                new QueryWrapper<HostsGroups>()
                        .select("groupid")
                        .eq("hostid", hostid));
        if (hostsGroups.size() > 0) {
            ArrayList<Long> hostsGroupids = new ArrayList<>();
            for (HostsGroups i : hostsGroups) {
                hostsGroupids.add(i.getGroupid());
            }
            List<Hstgrp> hstgrps = hstgrpMapper.selectList(
                    new QueryWrapper<Hstgrp>()
                            .eq("type", isTemplate ? Config.host.HOST_GROUP_TYPE_TEMPLATE_GROUP : Config.host.HOST_GROUP_TYPE_HOST_GROUP)
                            .in("groupid", hostsGroupids));
            resHost.setGroups(hstgrps);
        }

        // 主机模板列表
        List<HostsTemplates> hostsTemplates = hostsTemplatesMapper.selectList(new QueryWrapper<HostsTemplates>().eq("hostid", hostid));
        if (hostsTemplates.size() > 0) {
            ArrayList<Long> hostsTemplateids = new ArrayList<>();
            for (HostsTemplates i : hostsTemplates) {
                hostsTemplateids.add(i.getTemplateid());
            }
            List<Map<String, Object>> hostsTemplateList = hostsMapper.selectMaps(
                    new QueryWrapper<Hosts>()
                            .select("hostid,name")
                            .in("hostid", hostsTemplateids)
                            .eq("status", Config.host.HOST_STATUS_TEMPLATE));
            resHost.setTemplates(hostsTemplateList);
        }

        // 主机标签列表
        List<HostTag> hostTags = hostTagMapper.selectList(new QueryWrapper<HostTag>().eq("hostid", hostid));
        resHost.setTags(hostTags);

        // 主机宏列表
        List<Hostmacro> hostmacros = hostmacroMapper.selectList(new QueryWrapper<Hostmacro>().eq("hostid", hostid));
        resHost.setMacros(hostmacros);

        // 主机值映射列表
        List<Valuemap> valuemaps = valuemapMapper.selectList(new QueryWrapper<Valuemap>().eq("hostid", hostid));
        if (valuemaps.size() > 0) {
            ArrayList<Long> valuemapids = new ArrayList<>();
            for (Valuemap i : valuemaps) {
                valuemapids.add(i.getValuemapid());
            }
            List<ValuemapMapping> valuemapList = valuemapMappingMapper.selectList(
                    new QueryWrapper<ValuemapMapping>()
                            .in("valuemapid", valuemapids)
                            .orderByAsc("sortorder"));

            for (Valuemap i : valuemaps) {
                ArrayList<ValuemapMapping> valuemap_mapping = new ArrayList<>();
                for (ValuemapMapping ii : valuemapList) {
                    if (ii.getValuemapid().equals(i.getValuemapid())) {
                        valuemap_mapping.add(ii);
                    }
                }
                i.setMappings(valuemap_mapping);
            }
            resHost.setValuemaps(valuemaps);
        }

        return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT, resHost));
    }

    @Override
    public Result<JSONObject> detail_count(ArgDetail args) {

        // 主机和模板参数不同
        Boolean isTemplate = (null != args.kind && args.kind.equals(1)) ? true : false;

        Long hostid = args.hostid;
        QueryWrapper<Hosts> qw = new QueryWrapper<>();
        if (isTemplate) {
            qw.eq("status", Config.host.HOST_STATUS_TEMPLATE);
        } else {
            qw.in("status", Arrays.asList(Config.host.HOST_STATUS_MONITORED, Config.host.HOST_STATUS_NOT_MONITORED));
        }
        qw.eq("hostid", hostid);
        Integer count = hostsMapper.selectCount(qw);
        if (!count.equals(1)) {
            return Result.fail(HttpStatusEnum.BAD_REQUEST, "无效id");
        }
        HashMap<String, Integer> result = new HashMap<>();

        // 监控项个数
        QueryWrapper<Items> items_qw = new QueryWrapper<>();
        items_qw.groupBy("hostid");
        items_qw.ne("type", Config.item.ITEM_TYPE_HTTPTEST);
        items_qw.in("flags", Arrays.asList(Config.host.ZBX_FLAG_DISCOVERY_NORMAL, Config.host.ZBX_FLAG_DISCOVERY_CREATED));
        items_qw.eq("hostid", hostid);

        Integer items_count = itemsMapper.selectCount(items_qw);
        result.put("items", items_count);

        // 触发器个数
        QueryWrapper<Triggers> trigger_qw = new QueryWrapper<>();
        trigger_qw.eq("i.hostid", hostid);
        trigger_qw.in("t.flags", Arrays.asList(Config.host.ZBX_FLAG_DISCOVERY_NORMAL, Config.host.ZBX_FLAG_DISCOVERY_CREATED));
        List<Map> triggers = triggersMapper.countHostTrigger(trigger_qw);
        if (triggers.size() == 1) {
            result.put("triggers", Integer.parseInt(triggers.get(0).get("rowscount").toString()));
        } else {
            result.put("triggers", 0);
        }

        // 图表个数
        QueryWrapper<Graphs> graphs_qw = new QueryWrapper<>();
        graphs_qw.eq("i.hostid", hostid);
        graphs_qw.in("g.flags", Arrays.asList(Config.host.ZBX_FLAG_DISCOVERY_NORMAL, Config.host.ZBX_FLAG_DISCOVERY_CREATED));
        List<Map> graphs = graphsMapper.countHostGraphs(graphs_qw);
        if (graphs.size() == 1) {
            result.put("graphs", Integer.parseInt(graphs.get(0).get("rowscount").toString()));
        } else {
            result.put("graphs", 0);
        }

        // 自动发现规则个数
        QueryWrapper<Items> items_qw2 = new QueryWrapper<>();
        items_qw2.groupBy("hostid");
        items_qw2.in("flags", Arrays.asList(Config.host.ZBX_FLAG_DISCOVERY_RULE));
        items_qw2.eq("hostid", hostid);
        Integer items_count2 = itemsMapper.selectCount(items_qw2);
        result.put("discoveries", items_count2);


        return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT, result));
    }

    @Data
    public static class ArgCreate {
        @NotNull
        String host;
        String name;
        String description;
        ArrayList<HostInterface> interfaces;
        @NotNull
        ArrayList<ArgGroup> groups;
        ArrayList<ArgTemplate> templates;
        ArrayList<ArgTag> tags;
        ArrayList<ArgMacro> macros;
        ArrayList<ArgValuemap> valuemaps;
        Integer collectionid;
        Integer categoryid;
        Integer kind; // 1为模板，否则为主机
    }

    @Data
    public static class ArgTag {
        String tag;
        String value;
    }

    @Data
    public static class ArgGroup {
        Long groupid;
    }

    @Data
    public static class ArgTemplate {
        Long templateid;
    }

    @Data
    public static class ArgMacro {
        String macro;
        String value;
        String description;
        Integer type;   //    0 - (default) 文本宏;   1 - 密文宏;    2 - 密钥宏.
    }

    @Data
    public static class ArgValuemap {
        String name;
        @NotNull
        ArrayList<ArgValuemapMapping> mappings;
    }

    @Data
    public static class ArgValuemapMapping {
        @NotNull
        Integer type;
        @NotNull
        String value;
        @NotNull
        String newvalue;
        //      Integer sortorder;
    }

    @Override
    public Result<JSONObject> create(ArgCreate args) {

        // 主机和模板参数不同
        Boolean isTemplate = (null != args.kind && args.kind.equals(1)) ? true : false;

        // 校验valuemap
        ArrayList<ArgValuemap> argValuemaps = args.valuemaps;
        List<Integer> mapping_types = Arrays.asList(
                Config.host.VALUEMAP_MAPPING_TYPE_EQUAL,
                Config.host.VALUEMAP_MAPPING_TYPE_GREATER_EQUAL,
                Config.host.VALUEMAP_MAPPING_TYPE_LESS_EQUAL,
                Config.host.VALUEMAP_MAPPING_TYPE_IN_RANGE,
                Config.host.VALUEMAP_MAPPING_TYPE_REGEXP,
                Config.host.VALUEMAP_MAPPING_TYPE_DEFAULT);
        for (ArgValuemap v : argValuemaps) {
            ArrayList<ArgValuemapMapping> mappings = v.mappings;
            if (null == v.name || "" == v.name) {
                return Result.fail(HttpStatusEnum.BAD_REQUEST, "映射名称不能为空");
            }
            if (null == mappings || mappings.size() == 0) {
                return Result.fail(HttpStatusEnum.BAD_REQUEST, "映射不能为空");
            }
            for (ArgValuemapMapping mapping : mappings) {
                if (null == mapping.type || !mapping_types.contains(mapping.type)) {
                    return Result.fail(HttpStatusEnum.BAD_REQUEST, "type不能为空");
                }
                if (null == mapping.value || "" == mapping.value) {
                    return Result.fail(HttpStatusEnum.BAD_REQUEST, "值不能为空");
                }
                if (null == mapping.newvalue || "" == mapping.newvalue) {
                    return Result.fail(HttpStatusEnum.BAD_REQUEST, "映射值不能为空");
                }
            }
        }

        JSONObject params = new JSONObject();
        if (!isTemplate) {
            if (null == args.collectionid || args.collectionid <= 0) {
                return Result.fail(HttpStatusEnum.BAD_REQUEST, "应用集合不能为空");
            }
            if (null == args.categoryid || args.categoryid <= 0) {
                return Result.fail(HttpStatusEnum.BAD_REQUEST, "分类不能为空");
            }
            params.put("interfaces", args.getInterfaces());
        }
        params.put("host", args.host);
        params.put("name", (null == args.name || args.name == "") ? args.host : args.name);
        params.put("description", args.description);
        params.put("groups", args.getGroups());
        params.put("templates", args.getTemplates());
        params.put("tags", args.tags);
        params.put("macros", args.macros);

        JSONObject res = ZabbixApi.post(isTemplate ? "template.create" : "host.create", Config.AUTH, params);

        if (!isTemplate) {
            JSONObject result = res.getJSONObject("result");
            if (null != result) {
                JSONArray hostids = result.getJSONArray("hostids");

                ArrayList<Integer> ids = new ArrayList<>();
                for (Object o : hostids) {
                    int i = Integer.parseInt((String) o);
                    ids.add(i);
                }
                if (ids.size() > 0) {
                    UpdateWrapper<Hosts> uw = new UpdateWrapper<>();
                    uw.in("hostid", ids);
                    Integer collectionid = args.collectionid;
                    Integer count_coll = collectionMapper.selectCount(new QueryWrapper<MsHostCollection>().eq("id", collectionid));
                    if (count_coll == 1) {
                        uw.set("collectionid", collectionid);
                        uw.set("categoryid", args.categoryid);
                        hostsMapper.update(null, uw);
                    }
                }
            }
        }

        return Result.success(res);
    }


    @Data
    public static class ArgDelete {
        ArrayList<Long> hostids;
        Integer kind; // 1为模板，否则为主机
    }

    @Override
    public Result<JSONObject> delete(ArgDelete args) {
        JSONObject res = ZabbixApi.post((null != args.kind && args.kind == 1) ? "template.delete" : "host.delete", Config.AUTH, args.hostids);
        return Result.success(res);
    }

    @Data
    public static class ArgUpdate {
        @NotNull(message = "主机id不能为空")
        Long hostid;
        Integer status; // 0开启 1关闭
        Integer collectionid;
        Integer categoryid;
        String host;
        String name;
        String description;
        ArrayList<HostInterface> interfaces;
        ArrayList<ArgGroup> groups;
        ArrayList<ArgTemplate> templates;
        ArrayList<ArgTag> tags;
        ArrayList<ArgMacro> macros;
        ArrayList templates_clear;
        Integer kind; // 1为模板，否则为主机
    }

    @Override
    public Result<JSONObject> update(ArgUpdate args) {

        // 主机和模板参数不同
        Boolean isTemplate = (null != args.kind && args.kind.equals(1)) ? true : false;

        JSONObject params = new JSONObject();

        if (!isTemplate) {
            params.put("hostid", args.hostid);
            params.put("interfaces", args.getInterfaces());
        } else {
            params.put("templateid", args.hostid);
        }
        params.put("status", args.status);
        params.put("host", args.host);
        params.put("name", args.name);
        params.put("description", args.description);
        params.put("groups", args.getGroups());
        params.put("templates", args.getTemplates());
        params.put("tags", args.tags);
        params.put("macros", args.macros);
        params.put("templates_clear", args.templates_clear);

        JSONObject res = ZabbixApi.post(isTemplate ? "template.update" : "host.update", Config.AUTH, params);

        if (!isTemplate) {
            JSONObject result = res.getJSONObject("result");
            if (null != result) {
                UpdateWrapper<Hosts> uw = new UpdateWrapper<>();
                uw.eq("hostid", args.hostid);
                Integer collectionid = args.collectionid;
                Integer count_coll = 0;
                if (null != collectionid && collectionid > 0) {
                    count_coll = collectionMapper.selectCount(new QueryWrapper<MsHostCollection>().eq("id", collectionid));
                    if (count_coll == 1) {
                        uw.set("collectionid", collectionid);
                        uw.set("categoryid", args.categoryid);
                        hostsMapper.update(null, uw);
                    }
                }
            }
        }

        return Result.success(res);
    }

    @Data
    public static class ArgCount {
        @NotNull(message = "应用集合id不能为空")
        int collectionid;
    }

    @Override
    public Result<JSONObject> collectionList() {
        return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT, collectionMapper.selectList(null)));
    }

    @Override
    public Result<ArrayList<JSONObject>> count(ArgCount args) {
        return Result.success(collectionMapper.count(args.collectionid));
    }

    @Data
    public static class ArgValuemapGet {
        @NotNull
        Long hostid;
    }

    @Override
    public Result<JSONObject> valuemapList(ArgValuemapGet args) {
        Long hostid = args.getHostid();
        if (null == hostid) {
            return Result.fail(HttpStatusEnum.BAD_REQUEST, "参数不足");
        }
        List<Valuemap> result = valuemapMapper.selectList(new QueryWrapper<Valuemap>().eq("hostid", hostid));
        if (result.size() > 0) {
            ArrayList<Long> valuemapids = new ArrayList<>();
            for (Valuemap i : result) {
                valuemapids.add(i.getValuemapid());
            }
            List<ValuemapMapping> valuemapList = valuemapMappingMapper.selectList(
                    new QueryWrapper<ValuemapMapping>()
                            .in("valuemapid", valuemapids)
                            .orderByAsc("sortorder"));

            for (Valuemap i : result) {
                ArrayList<ValuemapMapping> valuemap_mapping = new ArrayList<>();
                for (ValuemapMapping ii : valuemapList) {
                    if (ii.getValuemapid().equals(i.getValuemapid())) {
                        valuemap_mapping.add(ii);
                    }
                }
                i.setMappings(valuemap_mapping);
            }
        }
        return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT, result));
    }

    @Data
    public static class ArgValuemapCreate {
        @NotNull(message = "主机不能为空")
        Long hostid;
        @NotNull(message = "值映射不能为空")
        ArrayList<ArgValuemap> valuemaps;
    }


    @Override
    @Transactional
    public Result<JSONObject> valuemapCreate(ArgValuemapCreate args) {
        Long hostid = args.hostid;
        Hosts hosts = hostsMapper.selectOne(
                new QueryWrapper<Hosts>()
                        .eq("hostid", hostid)
                        .in("status", Arrays.asList(Config.host.HOST_STATUS_MONITORED, Config.host.HOST_STATUS_NOT_MONITORED, Config.host.HOST_STATUS_TEMPLATE))
                        .in("flags", Arrays.asList(Config.host.ZBX_FLAG_DISCOVERY_NORMAL, Config.host.ZBX_FLAG_DISCOVERY_CREATED)).last("limit 1"));
        if (null == hosts) {
            return Result.fail(HttpStatusEnum.BAD_REQUEST, "主机错误");
        }
        // 校验valuemap
        ArrayList<ArgValuemap> argValuemaps = args.valuemaps;
        List<Integer> mapping_types = Arrays.asList(
                Config.host.VALUEMAP_MAPPING_TYPE_EQUAL,
                Config.host.VALUEMAP_MAPPING_TYPE_GREATER_EQUAL,
                Config.host.VALUEMAP_MAPPING_TYPE_LESS_EQUAL,
                Config.host.VALUEMAP_MAPPING_TYPE_IN_RANGE,
                Config.host.VALUEMAP_MAPPING_TYPE_REGEXP,
                Config.host.VALUEMAP_MAPPING_TYPE_DEFAULT);
        for (ArgValuemap v : argValuemaps) {
            ArrayList<ArgValuemapMapping> mappings = v.mappings;
            if (null == v.name || "" == v.name) {
                return Result.fail(HttpStatusEnum.BAD_REQUEST, "映射名称不能为空");
            }
            if (null == mappings || mappings.size() == 0) {
                return Result.fail(HttpStatusEnum.BAD_REQUEST, "映射不能为空");
            }
            for (ArgValuemapMapping mapping : mappings) {
                if (null == mapping.type || !mapping_types.contains(mapping.type)) {
                    return Result.fail(HttpStatusEnum.BAD_REQUEST, "type不能为空");
                }
                if (null == mapping.value || "" == mapping.value) {
                    return Result.fail(HttpStatusEnum.BAD_REQUEST, "值不能为空");
                }
                if (null == mapping.newvalue || "" == mapping.newvalue) {
                    return Result.fail(HttpStatusEnum.BAD_REQUEST, "映射值不能为空");
                }
            }
        }
        // 添加valuemap
        ArrayList<String> names = new ArrayList<String>();
        for (ArgValuemap v : argValuemaps) {
            names.add(v.getName());
        }
        HashSet<String> names_set = new HashSet<>(names);
        if (names_set.size() < names.size()) {
            return Result.fail(HttpStatusEnum.BAD_REQUEST, "值映射名称重复");
        }
        Valuemap name = valuemapMapper.selectOne(new QueryWrapper<Valuemap>().in("name", names).eq("hostid", hostid).last("limit 1"));
        if (null != name) {
            return Result.fail(HttpStatusEnum.BAD_REQUEST, "已存在相同的值映射名称");
        }
        Ids ids = idsMapper.selectOne(new QueryWrapper<Ids>().eq("table_name", "valuemap").eq("field_name", "valuemapid"));
        Long nextid = ids.getNextid();

        ArrayList<Valuemap> valuemaps = new ArrayList<>();
        for (int i = 0; i < argValuemaps.size(); i++) {
            Valuemap valuemap = new Valuemap();
            nextid += 1;
            String uuid = Common.getRandomString(11, null) + Config.CUSTOM_PARAM + System.currentTimeMillis();
            valuemap.setValuemapid(nextid)
                    .setHostid(hostid)
                    .setName(argValuemaps.get(i).getName())
                    .setUuid(uuid);
            valuemaps.add(valuemap);
        }
        valuemapMapper.save(valuemaps);
        idsMapper.update(null, new UpdateWrapper<Ids>().eq("table_name", "valuemap").eq("field_name", "valuemapid").set("nextid", nextid));

        Ids ids1 = idsMapper.selectOne(new QueryWrapper<Ids>().eq("table_name", "valuemap_mapping").eq("field_name", "valuemap_mappingid"));
        Long nextid1 = ids1.getNextid();
        ArrayList<ValuemapMapping> valuemapMappings = new ArrayList<>();
        for (int i = 0; i < argValuemaps.size(); i++) {
            ArrayList<ArgValuemapMapping> mappings = argValuemaps.get(i).mappings;
            for (int i1 = 0; i1 < mappings.size(); i1++) {
                ValuemapMapping valuemapMapping = new ValuemapMapping();
                nextid1 += 1;
                valuemapMapping
                        .setValuemap_mappingid(nextid1)
                        .setType(mappings.get(i1).getType())
                        .setValuemapid(valuemaps.get(i).getValuemapid())
                        .setValue(mappings.get(i1).getValue())
                        .setNewvalue(mappings.get(i1).getNewvalue())
                        .setSortorder(i1);
                valuemapMappings.add(valuemapMapping);
            }
        }
        valuemapMappingMapper.save(valuemapMappings);
        idsMapper.update(null, new UpdateWrapper<Ids>().eq("table_name", "valuemap_mapping").eq("field_name", "valuemap_mappingid").set("nextid", nextid1));

        return Result.success();
    }

    @Data
    public static class ArgFastAdd {
        @NotNull(message = "主机名不能为空")
        String host;
        String name;
        @NotNull(message = "集合不能为空")
        Long collectionid;
        @NotNull(message = "分组不能为空")
        ArrayList<ArgGroup> groups;
        @NotNull(message = "类型不能为空")
        Long categoryid;
        @NotNull(message = "IP不能为空")
        String ip;
        @NotNull(message = "端口不能为空")
        String port;
        String description;
        ArrayList<ArgMacro> macros;
    }

    @Override
    public Result fastAdd(ArgFastAdd args) {
        HostCategoryEnum hostCategoryEnum = HostCategoryEnum.valueOf(args.categoryid);
        if (null == hostCategoryEnum) {
            return Result.fail(HttpStatusEnum.BAD_REQUEST, "主机分类不可用");
        }

        JSONObject params = new JSONObject();

        ArrayList<HostInterface> interfaces = new ArrayList<>();
        HostInterface hi = new HostInterface();
        hi.setMain(Config.host.INTERFACE_PRIMARY);
        hi.setUseip(Config.host.INTERFACE_USE_IP);
        hi.setType(hostCategoryEnum.interfaceType());
        hi.setIp(args.ip);
        hi.setPort(args.port);
        hi.setDns("");
        interfaces.add(hi);
        params.put("interfaces", interfaces);

        params.put("host", args.host);
        params.put("name", (null == args.name || args.name == "") ? args.host : args.name);
        params.put("description", args.description);
        params.put("groups", args.getGroups());
        params.put("macros", args.macros);

        ArrayList<ArgTemplate> templates = new ArrayList<>();
        ArgTemplate argTemplate = new ArgTemplate();
        argTemplate.setTemplateid(hostCategoryEnum.templateid());
        templates.add(argTemplate);
        params.put("templates", templates);

        JSONObject res = ZabbixApi.post("host.create", Config.AUTH, params);
        JSONObject result = res.getJSONObject("result");
        if (null != result) {
            JSONArray hostids = result.getJSONArray("hostids");

            ArrayList<Integer> ids = new ArrayList<>();
            for (Object o : hostids) {
                int i = Integer.parseInt((String) o);
                ids.add(i);
            }
            if (ids.size() > 0) {
                UpdateWrapper<Hosts> uw = new UpdateWrapper<>();
                uw.in("hostid", ids);
                Long collectionid = args.collectionid;
                Integer count_coll = collectionMapper.selectCount(new QueryWrapper<MsHostCollection>().eq("id", collectionid));
                if (count_coll == 1) {
                    uw.set("collectionid", collectionid);
                    uw.set("categoryid", args.categoryid);
                    hostsMapper.update(null, uw);
                }
            }
        }

        return Result.success(res);
    }

    @Data
    public static class ArgAgentInstall {
        String ip;
        String username;
        String password;
        String port;
        String serverIp;
    }


    @Autowired
    private AsyncTaskService asyncTask;

    public Result agentInstall(ArgAgentInstall args) {

        Boolean scp = LinuxUtil.scp(args.ip, args.username, args.password);
        if (!scp) {
            return Result.fail(HttpStatusEnum.UNSTABLE_EXCEPTION, "文件传输失败");
        }

        // 添加到主机
        ArgCreate argCreate = new ArgCreate();
        argCreate.host = args.ip;

        // 主机群组 linux
        argCreate.groups = new ArrayList<>();
        ArgGroup hostGroup = new ArgGroup();
        hostGroup.setGroupid(Long.getLong("2"));
        argCreate.groups.add(hostGroup);

        // 主机模板 linux
        argCreate.templates = new ArrayList<>();
        ArgTemplate template = new ArgTemplate();
        template.setTemplateid(Long.getLong("10001"));
        argCreate.templates.add(template);

        // 主机接口
        argCreate.interfaces = new ArrayList<>();
        HostInterface hostInterface = new HostInterface();
        hostInterface.setIp(args.ip);
        hostInterface.setPort("10073");
        hostInterface.setMain(1);
        hostInterface.setDns("");
        hostInterface.setType(1);
        hostInterface.setUseip(1);
        argCreate.interfaces.add(hostInterface);

        JSONObject res_create = (JSONObject) this.create(argCreate).getData();
        if (null != res_create.get("error")) {
            return Result.fail(HttpStatusEnum.UNSTABLE_EXCEPTION, "创建主机失败");
        }

        asyncTask.agentInstall(args);

//         ZabbixAgentInstaller.agentInstall(args.ip, args.username, args.password, args.serverIp);
        return Result.success("", "安装正在进行中");
    }

}
