package com.zh.test_zabbix.serviceImpl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zh.test_zabbix.config.Config;
import com.zh.test_zabbix.mapper.*;
import com.zh.test_zabbix.mapper.res.*;
import com.zh.test_zabbix.model.*;
import com.zh.test_zabbix.service.IItemsService;
import com.zh.test_zabbix.serviceImpl.util.ServiceUtil;
import com.zh.test_zabbix.utils.Common;
import com.zh.test_zabbix.utils.ZabbixApi;
import com.zh.test_zabbix.utils.result.HttpStatusEnum;
import com.zh.test_zabbix.utils.result.Result;
import lombok.Data;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author zh
 * @since 2023-10-13
 */
@Service
public class ItemsServiceImpl extends ServiceImpl<ItemsMapper, Items> implements IItemsService {

    @Resource
    private ItemsMapper itemsMapper;

    @Resource
    private ItemTagMapper itemTagMapper;

    @Resource
    private HostsMapper hostsMapper;

    @Resource
    private ValuemapMapper valuemapMapper;

    @Resource
    private ValuemapMappingMapper valuemapMappingMapper;

    @Resource
    private HistoryMapper historyMapper;

    @Resource
    private FunctionsMapper functionsMapper;

    @Resource
    private ItemPreprocMapper itemPreprocMapper;

    @Resource
    private LldMacroPathMapper lldMacroPathMapper;

    @Resource
    private ItemDiscoveryMapper itemDiscoveryMapper;

    @Resource
    private TriggersMapper triggersMapper;

    @Data
    public static class ArgGet {
        Integer pageNum;
        Integer pageSize;
        ArrayList<Long> hostids;
        ArrayList<Long> itemids;
        Boolean selectTriggers;
        Boolean selectTags;
        Boolean selectMasterItem;
        Boolean selectTemplate;
        Boolean selectPreprocessing;

        Integer kind; // 1为模板，否则为主机
    }


    @Override
    public Result<JSONObject> get(ArgGet args) {

        // 主机和模板参数不同
        Boolean isTemplate = (null != args.kind && args.kind.equals(1)) ? true : false;


        QueryWrapper<Items> qw = new QueryWrapper<>();
        qw.ne("type", Config.item.ITEM_TYPE_HTTPTEST);
        qw.in("flags", Arrays.asList(Config.host.ZBX_FLAG_DISCOVERY_NORMAL, Config.host.ZBX_FLAG_DISCOVERY_CREATED));
        if (isTemplate) {
            qw.inSql("hostid", "SELECT hostid from hosts where status = " + Config.host.HOST_STATUS_TEMPLATE);
        } else {
            qw.inSql("hostid", "SELECT hostid from hosts where status <> " + Config.host.HOST_STATUS_TEMPLATE);
        }


        if (null != args.getHostids() && args.getHostids().size() > 0) {
            qw.in("hostid", args.getHostids());
        }
        if (null != args.getItemids() && args.getItemids().size() > 0) {
            qw.in("itemid", args.getItemids());
        }
        Integer count = itemsMapper.selectCount(qw);

        if (count == 0) {
            return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT, new ArrayList<>(), count));
        }

        ArrayList<Long> itemids = new ArrayList<>();
        qw.select("itemid");
        qw.orderByAsc("name_upper");
        ServiceUtil.Page page = ServiceUtil.init_page(args.pageNum, args.pageSize,count);
        qw.last("LIMIT " + (page.getNum() - 1) * page.getSize() + ", " + page.getSize());
        List<Items> items = itemsMapper.selectList(qw);
        for (Items item : items) {
            itemids.add(item.getItemid());
        }

        List<ResItemList> result = itemsMapper.itemList(new QueryWrapper<Items>().in("i.itemid", itemids).orderByAsc("i.name_upper"));

        // 上层自动发现
    if(!isTemplate) {
        ArrayList<ResItemDiscovery> itemDiscoveries = itemDiscoveryMapper.list(new QueryWrapper<ItemDiscovery>()
                .in("id1.itemid", itemids)
                .eq("i.flags", Config.host.ZBX_FLAG_DISCOVERY_CREATED));
        if (itemDiscoveries.size() > 0) {
            ArrayList<Long> parent_itemids = new ArrayList<>();
            for (ResItemDiscovery i : itemDiscoveries) {
                parent_itemids.add(i.getParent_itemid());
            }
            List<Items> discoveries = itemsMapper.selectList(new QueryWrapper<Items>()
                    .in("itemid", parent_itemids)
                    .eq("flags", Config.host.ZBX_FLAG_DISCOVERY_RULE));
            if (discoveries.size() > 0) {
                for (ResItemList r : result) {
                    for (ResItemDiscovery id : itemDiscoveries) {
                        if (id.getItemid().equals(r.getItemid())) {
                            for (Items d : discoveries) {
                                if (d.getItemid().equals(id.getParent_itemid())) {
                                    r.setDiscovery_ruleid(d.getItemid());
                                    r.setDiscovery_rule(d.getName());
                                    break;
                                }
                            }
                            break;
                        }
                    }
                }
            }
        }
    }

        // master_item
        if (null != args.getSelectMasterItem() && args.getSelectMasterItem()) {
            ArrayList<Long> master_itemids = new ArrayList<>();
            for (ResItemList r : result) {
                if (null != r.getMaster_itemid()) {
                    master_itemids.add(r.getMaster_itemid());
                }
            }
            if (master_itemids.size() > 0) {
                List<Items> master_items = itemsMapper.selectList(new QueryWrapper<Items>().select("itemid,name").in("itemid", master_itemids));
                for (ResItemList r : result) {
                    for (Items i : master_items) {
                        if (i.getItemid().equals(r.getMaster_itemid())) {
                            r.setMaster_item(i.getName());
                            break;
                        }
                    }
                }
            }
        }

        // item_template
        if (null != args.getSelectTemplate() && args.getSelectTemplate()) {
            ArrayList<Long> template_itemids = new ArrayList<>();
            for (ResItemList r : result) {
                if (null != r.getTemplateid()) {
                    template_itemids.add(r.getTemplateid());
                }
            }
            if (template_itemids.size() > 0) {
                List<Items> template_items = itemsMapper.selectList(
                        new QueryWrapper<Items>()
                                .select("itemid,hostid")
                                .in("flags", Arrays.asList(Config.host.ZBX_FLAG_DISCOVERY_NORMAL, Config.host.ZBX_FLAG_DISCOVERY_CREATED))
                                .in("itemid", template_itemids));
                ArrayList<Long> template_hostids = new ArrayList<>();
                for (Items r : template_items) {
                    if (null != r.getHostid()) {
                        template_hostids.add(r.getHostid());
                    }
                }
                if (template_hostids.size() > 0) {
                    List<Hosts> template_hosts = hostsMapper.selectList(
                            new QueryWrapper<Hosts>()
                                    .select("hostid,name")
                                    .eq("status", Config.host.HOST_STATUS_TEMPLATE)
                                    .in("hostid", template_hostids));
                    for (ResItemList r : result) {
                        for (Items ti : template_items) {
                            if (ti.getItemid().equals(r.getTemplateid())) {
                                for (Hosts h : template_hosts) {
                                    if (h.getHostid().equals(ti.getHostid())) {
                                        r.setTemplate(h.getName());
                                        break;
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        if (null != args.getSelectTriggers() && args.getSelectTriggers()) {
            QueryWrapper<Items> trigger_qw = new QueryWrapper<>();
            trigger_qw.in("t.flags", Arrays.asList(Config.host.ZBX_FLAG_DISCOVERY_NORMAL, Config.host.ZBX_FLAG_DISCOVERY_CREATED));
            trigger_qw.in("f.itemid", itemids);
            List<ResItemTriggerList> resItemTriggerList = itemsMapper.itemTriggerList(trigger_qw);

            //  处理expression部分的macro
            ArrayList<Integer> functionids = new ArrayList<>();
            for (ResItemTriggerList i : resItemTriggerList) {
                Pattern pattern = Pattern.compile("\\{(\\d+)\\}");
                Matcher matcher = pattern.matcher(i.getExpression());
                while (matcher.find()) {
                    Integer number = Integer.valueOf(matcher.group().replace("{", "").replace("}", ""));
                    functionids.add(number);
                }
            }
            if (functionids.size() > 0) {

                List<Functions> functions = functionsMapper.selectList(new QueryWrapper<Functions>().in("functionid", functionids));
                ArrayList<Long> hostids = new ArrayList<>();
                for (ResItemList r : result) {
                    hostids.add(r.getHostid());
                }
                List<Hosts> hosts = hostsMapper.selectList(new QueryWrapper<Hosts>().select("host,hostid").in("hostid", hostids));

                for (ResItemTriggerList i : resItemTriggerList) {
                    String expression = i.getExpression();
                    String expression_html = i.getExpression();
                    Pattern pattern = Pattern.compile("\\{(\\d+)\\}");
                    Matcher matcher = pattern.matcher(expression);
                    while (matcher.find()) {
                        String str = matcher.group();
                        for (Functions f : functions) {
                            if (f.getFunctionid().equals(Long.valueOf(str.replace("{", "").replace("}", "")))) {
                                for (ResItemList r : result) {
                                    if (r.getItemid().equals(f.getItemid())) {
                                        for (Hosts h : hosts) {
                                            if (h.getHostid().equals(r.getHostid())) {
                                                String parameter = f.getParameter();
                                                parameter = parameter.replace("$", "/" + h.getHost() + "/" + r.getKey_());
                                                String parameter_html = "<a " + "class='link'" + "data-itemid='" + r.getItemid() + "'>" + parameter + "</a>";
                                                expression_html = expression_html.replace(str, f.getName() + "(" + parameter_html + ")");
                                                expression = expression.replace(str, f.getName() + "(" + parameter + ")");
                                                break;
                                            }
                                        }
                                        break;
                                    }
                                }
                                break;
                            }
                        }
                    }
                    i.setExpression(expression);
                    i.setExpression_html(expression_html);
                }
            }

            for (ResItemList r : result) {
                r.setTriggers(resItemTriggerList.stream()
                        .filter(i -> i.getItemid().equals(r.getItemid()))
                        .collect(Collectors.toList()));
//                    ArrayList<ResItemTriggerList> triggerLists = new ArrayList<>();
//                    for (ResItemTriggerList ii : resItemTriggerList) {
//                        if (ii.getItemid().equals(r.getItemid())) {
//                            String expression = ii.getExpression();
//                            ii.setExpression(expression.replaceFirst("\\$",r.getKey_()));
//                            triggerLists.add(ii);
//                        }
//                    }
//                    r.setTriggers(triggerLists);
            }
        }

        // 标签
        if (null != args.getSelectTags() && args.getSelectTags()) {
            List<ItemTag> itemTags = itemTagMapper.selectList(new QueryWrapper<ItemTag>().in("itemid", itemids));
            for (ResItemList r : result) {
                r.setTags(itemTags.stream()
                        .filter(i -> i.getItemid().equals(r.getItemid()))
                        .collect(Collectors.toList()));
            }
        }

        // 预处理
        if (null != args.getSelectPreprocessing() && args.getSelectPreprocessing()) {
            List<ItemPreproc> itemPreprocs = itemPreprocMapper.selectList(new QueryWrapper<ItemPreproc>().in("itemid", itemids));
            for (ResItemList r : result) {
                r.setPreprocessing(itemPreprocs.stream()
                        .filter(i -> i.getItemid().equals(r.getItemid()))
                        .collect(Collectors.toList()));
            }
        }

        return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT, result, count, page.getNum(), page.getSize()));
    }


    @Data
    public static class ArgData {
        Integer pageNum;
        Integer pageSize;
        ArrayList<Long> groupids;
        ArrayList<Long> hostids;
        ArrayList<Long> itemids;
        ArrayList<ArgSearchTag> tags;
        HashMap search;
        Boolean selectTags;
        Boolean selectHosts;
        Object sortfield;
        Boolean selectValuemaps;
    }

    @Data
    public static class ArgSearchTag {
        String tag;
        String value;
        Integer operator;  // 0 - (默认) Like（相似）；1 - Equal（相等）；2 - Not like（不相似）；3 - Not equal（不相等）；4 - Exists（存在）；5 - Not exists（不存在）
    }

    @Override
    public Result<JSONObject> data(ArgData args) {
        QueryWrapper<Items> qw = new QueryWrapper<>();
        qw.in("flags", Arrays.asList(Config.host.ZBX_FLAG_DISCOVERY_NORMAL, Config.host.ZBX_FLAG_DISCOVERY_CREATED));
        qw.eq("status", Config.item.ITEM_STATUS_ACTIVE);
        if (null != args.groupids && args.groupids.size() > 0) {
            qw.inSql("hostid", "SELECT hostid FROM hosts_groups WHERE groupid in (" + Common.getArrayList(args.groupids) + ")");
        }
        if (null != args.hostids && args.hostids.size() > 0) {
            qw.in("hostid", args.hostids);
        }
        if (null != args.itemids && args.itemids.size() > 0) {
            qw.in("itemid", args.itemids);
        }
        HashMap search = args.search;
        if (null != search) {
            if (null != search.get("name")) {
                String name = search.get("name").toString();
                if (null != name) {
                    qw.like("name_upper", name.toUpperCase());
                }
            }
        }
        Integer count = itemsMapper.selectCount(qw);
        if (count == 0) {
            return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT, new ArrayList<>(), count));
        }

        ArrayList<Long> itemids_ = new ArrayList<>();
        qw.select("itemid");
        ServiceUtil.Page page = ServiceUtil.init_page(args.pageNum, args.pageSize,count);
        qw.last("LIMIT " + (page.getNum() - 1) * page.getSize() + ", " + page.getSize());
        List<Items> items = itemsMapper.selectList(qw);
        for (Items item : items) {
            itemids_.add(item.getItemid());
        }

        QueryWrapper<Items> itemData_qw = new QueryWrapper<>();
        itemData_qw.in("i.flags", Arrays.asList(Config.host.ZBX_FLAG_DISCOVERY_NORMAL, Config.host.ZBX_FLAG_DISCOVERY_CREATED));
        itemData_qw.in("i.itemid", itemids_);
        List<ResItemData> result = itemsMapper.itemData(itemData_qw);

        if (null != args.getSelectHosts() && args.getSelectHosts()) {
            QueryWrapper<Hosts> hosts_qw = new QueryWrapper<>();
            hosts_qw.select("hostid", "name");
            hosts_qw.inSql("hostid", "SELECT DISTINCT hostid from items where itemid in (" + Common.getArrayList(itemids_) + ")");
            List<Hosts> hosts = hostsMapper.selectList(hosts_qw);
            for (ResItemData r : result) {
                for (Hosts host : hosts) {
                    if (host.getHostid().equals(r.getHostid())) {
                        r.setHost(host);
                        break;
                    }
                }

            }
        }

        // 添加itemTag
        if (null != args.getSelectTags() && args.getSelectTags()) {
            List<ItemTag> itemTags = itemTagMapper.selectList(new QueryWrapper<ItemTag>().in("itemid", itemids_));
            for (ResItemData r : result) {
                List<ItemTag> collect = itemTags.stream()
                        .filter(i -> i.getItemid().equals(r.getItemid()))
                        .collect(Collectors.toList());
                r.setTags(collect);
            }
        }

        // 添加valuemap
        if (null != args.selectValuemaps && args.selectValuemaps) {
            ArrayList<Long> valuemapids = new ArrayList<>();
            for (ResItemData r : result) {
                if (null != r.getValuemapid()) {
                    valuemapids.add(r.getValuemapid());
                }
            }
            if (valuemapids.size() > 0) {
                List<Valuemap> valuemaps = valuemapMapper.selectList(new QueryWrapper<Valuemap>().in("valuemapid", valuemapids));
                List<ValuemapMapping> valuemapList = valuemapMappingMapper.selectList(
                        new QueryWrapper<ValuemapMapping>()
                                .in("valuemapid", valuemapids)
                                .orderByAsc("sortorder"));

                for (Valuemap i : valuemaps) {
                    ArrayList<ValuemapMapping> valuemap_mapping = new ArrayList<>();
                    for (ValuemapMapping ii : valuemapList) {
                        if (ii.getValuemapid().equals(i.getValuemapid())) {
                            valuemap_mapping.add(ii);
                        }
                    }
                    i.setMappings(valuemap_mapping);
                }
                for (ResItemData r : result) {
                    for (Valuemap i : valuemaps) {
                        if (i.getValuemapid().equals(r.getValuemapid())) {
                            r.setValuemap(i);
                            break;
                        }
                    }
                }
            }
        }

        QueryWrapper his_qw;

        for (ResItemData r : result) {
            ArrayList<ResHistory> history = null;
            his_qw = new QueryWrapper<>();
            his_qw.eq("itemid", r.getItemid());
            his_qw.orderByDesc("clock","ns");
            his_qw.last("LIMIT 2");
            switch (r.getValue_type()) {
                case Config.item.ITEM_VALUE_TYPE_FLOAT:
                    history = historyMapper.history(his_qw);
                    break;
                case Config.item.ITEM_VALUE_TYPE_STR:
                    history = historyMapper.history_str(his_qw);
                    break;
                case Config.item.ITEM_VALUE_TYPE_LOG:
                    history = historyMapper.history_log(his_qw);
                    break;
                case Config.item.ITEM_VALUE_TYPE_UINT64:
                    history = historyMapper.history_uint(his_qw);
                    break;
                case Config.item.ITEM_VALUE_TYPE_TEXT:
                    history = historyMapper.history_text(his_qw);
                    break;
            }
            if (history.size() > 0) {
                r.setLastclock(history.get(0).getClock());
                r.setLastns(history.get(0).getNs());
                r.setLastvalue(history.get(0).getValue());
            }
            if (history.size() == 2) {
                r.setPrevvalue(history.get(1).getValue());
            }
        }

        return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT, result, count, page.getNum(), page.getSize()));
    }


    @Data
    public static class ArgDetail {
        Long itemid;
    }

    @Override
    public Result<JSONObject> detail(ArgDetail args) {
        Long itemid = args.itemid;
        QueryWrapper<Items> qw = new QueryWrapper<>();
        qw.eq("itemid", itemid);
        Integer count = itemsMapper.selectCount(qw);
        if (!count.equals(1)) {
            return Result.fail(HttpStatusEnum.BAD_REQUEST, "无效id");
        }
        Items item = itemsMapper.selectOne(qw);
        if (null != item.getMaster_itemid()) {
            Items master_item = itemsMapper.selectOne(
                    new QueryWrapper<Items>()
                            .select("itemid,name")
                            .in("flags", Arrays.asList(Config.host.ZBX_FLAG_DISCOVERY_NORMAL, Config.host.ZBX_FLAG_DISCOVERY_CREATED))
                            .eq("itemid", item.getMaster_itemid()));
            item.setMaster_item(master_item.getName());
        }
        if (null != item.getTemplateid()) {
            Items template_item = itemsMapper.selectOne(
                    new QueryWrapper<Items>()
                            .select("itemid,hostid")
                            .in("flags", Arrays.asList(Config.host.ZBX_FLAG_DISCOVERY_NORMAL, Config.host.ZBX_FLAG_DISCOVERY_CREATED))
                            .eq("itemid", item.getTemplateid()));
            Hosts template_host = hostsMapper.selectOne(
                    new QueryWrapper<Hosts>()
                            .select("hostid,name")
                            .eq("status", Config.host.HOST_STATUS_TEMPLATE)
                            .eq("hostid", template_item.getHostid()));
            item.setTemplate(template_host.getName());
        }
        List<Valuemap> valuemaps = valuemapMapper.selectList(new QueryWrapper<Valuemap>().eq("valuemapid", item.getValuemapid()));
        List<ItemTag> tags = itemTagMapper.selectList(new QueryWrapper<ItemTag>().eq("itemid", itemid));
        List<ItemPreproc> preprocs = itemPreprocMapper.selectList(new QueryWrapper<ItemPreproc>().eq("itemid", itemid));
        item.setValuemaps(valuemaps);
        item.setTags(tags);
        item.setPreprocessing(preprocs);
        return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT, item));
    }

    @Data
    public static class ArgCreate {
        @NotNull(message = "模板id不能为空")
        Long hostid;
        @NotNull(message = "名称不能为空")
        String name;
        @NotNull(message = "键值不能为空")
        String key_;
        @NotNull(message = "类型不能为空")
        Integer type;    // 0 - agent监控项; 18 - 依赖监控项; 21 - 脚本监控项
        @NotNull(message = "信息类型不能为空")
        Integer value_type;
        Long master_itemid;
        String units;
        @NotNull(message = "更新间隔不能为空")
        String delay;
        ArrayList<ArgTag> tags;
        ArrayList<ArgPreproc> preprocessing;
        String history;
        String trends;
        String description;
        Integer valuemapid;
    }

    @Data
    public static class ArgTag {
        String tag;
        String value;
    }

    @Data
    public static class ArgPreproc {
        Integer type;
        String params;
        Integer error_handler;
        String error_handler_params;
    }

    @Override
    public Result<JSONObject> create(ArgCreate args) {
        JSONObject params = new JSONObject();
        params.put("hostid", args.hostid);
        params.put("name", args.name);
        params.put("key_", args.key_);
        params.put("type", args.type);
        params.put("value_type", args.value_type);
        params.put("delay", args.delay);
        if (args.type.equals(Config.item.ITEM_TYPE_DEPENDENT)) {
            if (null == args.master_itemid) {
                return Result.fail(HttpStatusEnum.BAD_REQUEST, "缺少依赖item信息");
            }
            params.put("delay", 0);
            params.put("master_itemid", args.master_itemid);
        }
        params.put("units", args.units);
        params.put("tags", args.tags);
        params.put("preprocessing", args.preprocessing);
        params.put("history", args.history);
        params.put("trends", args.trends);
        params.put("description", args.description);
        params.put("valuemapid", args.valuemapid);

        return Result.success(ZabbixApi.post("item.create", Config.AUTH, params));
    }

    @Data
    public static class ArgDelete {
        ArrayList<Long> itemids;
    }

    @Override
    public Result<JSONObject> delete(ArgDelete args) {
        return Result.success(ZabbixApi.post("item.delete", Config.AUTH, args.itemids));
    }

    @Data
    public static class ArgUpdate {
        @NotNull(message = "监控项id不能为空")
        Long itemid;
        String name;
        String key_;
        Integer type;    // 0 - agent监控项; 18 - 依赖监控项; 21 - 脚本监控项
        Integer value_type;
        Long master_itemid;
        String units;
        String delay;
        ArrayList<ArgTag> tags;
        ArrayList<ArgPreproc> preprocessing;
        String history;
        String trends;
        String description;
        Integer valuemapid;
    }

    @Override
    public Result<JSONObject> update(ArgUpdate args) {
        JSONObject params = new JSONObject();
        params.put("itemid", args.itemid);
        params.put("name", args.name);
        params.put("key_", args.key_);
        params.put("type", args.type);
        params.put("value_type", args.value_type);
        params.put("delay", args.delay);
        if (args.type.equals(Config.item.ITEM_TYPE_DEPENDENT)) {
            if (null == args.master_itemid) {
                return Result.fail(HttpStatusEnum.BAD_REQUEST, "缺少依赖item信息");
            }
            params.put("delay", 0);
            params.put("master_itemid", args.master_itemid);
        }
        params.put("units", args.units);
        params.put("tags", args.tags);
        params.put("preprocessing", args.preprocessing);
        params.put("history", args.history);
        params.put("trends", args.trends);
        params.put("description", args.description);
        params.put("valuemapid", args.valuemapid);

        return Result.success(ZabbixApi.post("item.update", Config.AUTH, params));
    }

    @Data
    public static class ArgSwitchStatus {
        ArrayList<Long> itemids;
        @NotNull(message = "状态不能为空")
        @Min(value = 0, message = "0为开启;1为关闭")
        @Max(value = 1, message = "0为开启;1为关闭")
        Integer status;
    }

    @Override
    public Result<JSONObject> switchStatus(ArgSwitchStatus args) {
        UpdateWrapper<Items> uw = new UpdateWrapper<>();
        uw.set("status", args.status);
        uw.in("itemid", args.itemids);
        itemsMapper.update(null, uw);
        return Result.success(null);
    }


    @Data
    public static class ArgGetItemDiscovery {
        Integer pageNum;
        Integer pageSize;
        ArrayList<Long> hostids;
        ArrayList<Long> itemids;
        Boolean selectMasterItem;
        Boolean selectTemplate;
        String selectItems;
        String selectTriggers;
        String selectGraphs;

        Integer kind; // 1为模板，否则为主机
    }


    @Override
    public Result<JSONObject> getItemDiscovery(ArgGetItemDiscovery args) {

        // 主机和模板参数不同
        Boolean isTemplate = (null != args.kind && args.kind.equals(1)) ? true : false;

        QueryWrapper<Items> qw = new QueryWrapper<>();
        qw.eq("flags", Config.host.ZBX_FLAG_DISCOVERY_RULE);
        if (isTemplate) {
            qw.inSql("hostid", "SELECT hostid from hosts where status = " + Config.host.HOST_STATUS_TEMPLATE);
        } else {
            qw.inSql("hostid", "SELECT hostid from hosts where status <> " + Config.host.HOST_STATUS_TEMPLATE);
        }

        if (null != args.getHostids() && args.getHostids().size() > 0) {
            qw.in("hostid", args.getHostids());
        }
        if (null != args.getItemids() && args.getItemids().size() > 0) {
            qw.in("itemid", args.getItemids());
        }
        Integer count = itemsMapper.selectCount(qw);

        if (count == 0) {
            return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT, new ArrayList<>(), count));
        }

        ArrayList<Long> itemids = new ArrayList<>();
        qw.select("itemid");
        qw.orderByAsc("name_upper");
        ServiceUtil.Page page = ServiceUtil.init_page(args.pageNum, args.pageSize,count);
        qw.last("LIMIT " + (page.getNum() - 1) * page.getSize() + ", " + page.getSize());
        List<Items> items = itemsMapper.selectList(qw);
        for (Items item : items) {
            itemids.add(item.getItemid());
        }

        List<ResItemList> result = itemsMapper.itemList(new QueryWrapper<Items>().in("i.itemid", itemids).orderByAsc("i.name_upper"));

        // master_item
        if (null != args.getSelectMasterItem() && args.getSelectMasterItem()) {
            ArrayList<Long> master_itemids = new ArrayList<>();
            for (ResItemList r : result) {
                if (null != r.getMaster_itemid()) {
                    master_itemids.add(r.getMaster_itemid());
                }
            }
            if (master_itemids.size() > 0) {
                List<Items> master_items = itemsMapper.selectList(
                        new QueryWrapper<Items>()
                                .select("itemid,name")
                                .in("flags", Arrays.asList(Config.host.ZBX_FLAG_DISCOVERY_NORMAL, Config.host.ZBX_FLAG_DISCOVERY_CREATED))
                                .in("itemid", master_itemids));
                for (ResItemList r : result) {
                    for (Items i : master_items) {
                        if (i.getItemid().equals(r.getMaster_itemid())) {
                            r.setMaster_item(i.getName());
                            break;
                        }
                    }
                }
            }
        }

        // item_template
        if (null != args.getSelectTemplate() && args.getSelectTemplate()) {
            ArrayList<Long> template_itemids = new ArrayList<>();
            for (ResItemList r : result) {
                if (null != r.getTemplateid()) {
                    template_itemids.add(r.getTemplateid());
                }
            }
            if (template_itemids.size() > 0) {
                List<Items> template_items = itemsMapper.selectList(
                        new QueryWrapper<Items>()
                                .select("itemid,hostid")
                                .eq("flags", Config.host.ZBX_FLAG_DISCOVERY_RULE)
                                .in("itemid", template_itemids));
                ArrayList<Long> template_hostids = new ArrayList<>();
                for (Items r : template_items) {
                    if (null != r.getHostid()) {
                        template_hostids.add(r.getHostid());
                    }
                }
                if (template_hostids.size() > 0) {
                    List<Hosts> template_hosts = hostsMapper.selectList(
                            new QueryWrapper<Hosts>()
                                    .select("hostid,name")
                                    .eq("status", Config.host.HOST_STATUS_TEMPLATE)
                                    .in("hostid", template_hostids));
                    for (ResItemList r : result) {
                        for (Items ti : template_items) {
                            if (ti.getItemid().equals(r.getTemplateid())) {
                                for (Hosts h : template_hosts) {
                                    if (h.getHostid().equals(ti.getHostid())) {
                                        r.setTemplate(h.getName());
                                        break;
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }


        if (null != args.getSelectItems()) {
            QueryWrapper<ItemDiscovery> items_qw = new QueryWrapper<>();
            items_qw.groupBy("parent_itemid");
            items_qw.inSql("itemid", "SELECT itemid from items where flags = " + Config.host.ZBX_FLAG_DISCOVERY_PROTOTYPE);
            items_qw.in("parent_itemid", itemids);

            items_qw.select("COUNT(DISTINCT itemid) AS rowscount", "parent_itemid");

            List<Map<String, Object>> discoveryItems = itemDiscoveryMapper.selectMaps(items_qw);

            for (ResItemList r : result) {
                for (Map<String, Object> i : discoveryItems) {
                    if (Long.valueOf(i.get("parent_itemid").toString()).equals(r.getItemid())) {
                        r.setItems(i.get("rowscount"));
                        break;
                    }
                }
            }
        }


        if (null != args.getSelectTriggers()) {
            QueryWrapper<Triggers> trigger_qw = new QueryWrapper<>();
            trigger_qw.eq("t.flags", Config.host.ZBX_FLAG_DISCOVERY_PROTOTYPE);
            trigger_qw.in("id.parent_itemid", itemids);
            trigger_qw.groupBy("id.parent_itemid");
            List<Map> triggers = triggersMapper.countItemDiscoveryTrigger(trigger_qw);
            for (ResItemList r : result) {
                for (Map<String, Object> i : triggers) {
                    if (Long.valueOf(i.get("parent_itemid").toString()).equals(r.getItemid())) {
                        r.setTriggers(i.get("rowscount"));
                    }
                }
            }
        }

        return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT, result, count, page.getNum(), page.getSize()));
    }

    @Data
    public static class ArgItemDiscoveryDetail {
        Long itemid;
    }


    @Override
    public Result<JSONObject> itemDiscoveryDetail(ArgItemDiscoveryDetail args) {
        Long itemid = args.itemid;
        QueryWrapper<Items> qw = new QueryWrapper<>();
        qw.eq("flags", Config.host.ZBX_FLAG_DISCOVERY_RULE);
        qw.inSql("hostid", "SELECT hostid from hosts where status = " + Config.host.HOST_STATUS_TEMPLATE);
        qw.eq("itemid", itemid);
        Integer count = itemsMapper.selectCount(qw);

        if (!count.equals(1)) {
            return Result.fail(HttpStatusEnum.BAD_REQUEST, "无效id");
        }

        Items item = itemsMapper.selectOne(qw);

        if (null != item.getMaster_itemid()) {
            Items master_item = itemsMapper.selectOne(
                    new QueryWrapper<Items>()
                            .select("itemid,name")
                            .in("flags", Arrays.asList(Config.host.ZBX_FLAG_DISCOVERY_NORMAL, Config.host.ZBX_FLAG_DISCOVERY_CREATED))
                            .eq("itemid", item.getMaster_itemid()));
            item.setMaster_item(master_item.getName());
        }

        if (null != item.getTemplateid()) {
            Items template_item = itemsMapper.selectOne(
                    new QueryWrapper<Items>()
                            .select("itemid,hostid")
                            .eq("flags", Config.host.ZBX_FLAG_DISCOVERY_RULE)
                            .eq("itemid", item.getTemplateid()));
            Hosts template_host = hostsMapper.selectOne(
                    new QueryWrapper<Hosts>()
                            .select("hostid,name")
                            .eq("status", Config.host.HOST_STATUS_TEMPLATE)
                            .eq("hostid", template_item.getHostid()));
            item.setTemplate(template_host.getName());
        }

        List<ItemPreproc> preprocs = itemPreprocMapper.selectList(new QueryWrapper<ItemPreproc>().eq("itemid", itemid));
        item.setPreprocessing(preprocs);
        List<LldMacroPath> lldMacroPaths = lldMacroPathMapper.selectList(new QueryWrapper<LldMacroPath>().eq("itemid", itemid));
        item.setLld_macro_paths(lldMacroPaths);

        return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT, item));
    }

    @Data
    public static class ArgItemDiscoveryCreate {
        @NotNull(message = "模板id不能为空")
        Long hostid;
        @NotNull(message = "名称不能为空")
        String name;
        @NotNull(message = "键值不能为空")
        String key_;
        @NotNull(message = "类型不能为空")
        Integer type;    // 0 - agent监控项; 18 - 依赖监控项; 21 - 脚本监控项
        Long master_itemid;
        @NotNull(message = "更新间隔不能为空")
        String delay;
        ArrayList<ArgPreproc> preprocessing;
        ArrayList<ArgLldMacroPaths> lld_macro_paths;
        String description;
    }

    @Data
    public static class ArgLldMacroPaths {
        String lld_macro; // {#MACRO}
        String path;    // $.节点.路径
    }

    @Override
    public Result<JSONObject> itemDiscoveryCreate(ArgItemDiscoveryCreate args) {
        JSONObject params = new JSONObject();
        params.put("hostid", args.hostid);
        params.put("name", args.name);
        params.put("key_", args.key_);
        params.put("type", args.type);
        params.put("delay", args.delay);
        if (args.type.equals(Config.item.ITEM_TYPE_DEPENDENT)) {
            if (null == args.master_itemid) {
                return Result.fail(HttpStatusEnum.BAD_REQUEST, "缺少依赖item信息");
            }
            params.put("delay", 0);
            params.put("master_itemid", args.master_itemid);
        }
        params.put("preprocessing", args.preprocessing);
        params.put("lld_macro_paths", args.lld_macro_paths);
        params.put("description", args.description);

        return Result.success(ZabbixApi.post("discoveryrule.create", Config.AUTH, params));
    }


    @Data
    public static class ArgItemDiscoveriesDelete {
        ArrayList<Long> itemids;
    }

    @Override
    public Result<JSONObject> itemDiscoveriesDelete(ArgItemDiscoveriesDelete args) {
        return Result.success(ZabbixApi.post("discoveryrule.delete", Config.AUTH, args.itemids));
    }

    @Data
    public static class ArgItemDiscoveryUpdate {
        @NotNull(message = "监控项id不能为空")
        Long itemid;
        String name;
        String key_;
        Integer type;    // 0 - agent监控项; 18 - 依赖监控项; 21 - 脚本监控项
        Long master_itemid;
        String delay;
        ArrayList<ArgPreproc> preprocessing;
        ArrayList<ArgLldMacroPaths> lld_macro_paths;
        String description;
    }

    @Override
    public Result<JSONObject> itemDiscoveryUpdate(ArgItemDiscoveryUpdate args) {
        JSONObject params = new JSONObject();
        params.put("itemid", args.itemid);
        params.put("name", args.name);
        params.put("key_", args.key_);
        params.put("type", args.type);
        params.put("delay", args.delay);
        if (args.type.equals(Config.item.ITEM_TYPE_DEPENDENT)) {
            if (null == args.master_itemid) {
                return Result.fail(HttpStatusEnum.BAD_REQUEST, "缺少依赖item信息");
            }
            params.put("delay", 0);
            params.put("master_itemid", args.master_itemid);
        }
        params.put("preprocessing", args.preprocessing);
        params.put("lld_macro_paths", args.lld_macro_paths);
        params.put("description", args.description);

        return Result.success(ZabbixApi.post("discoveryrule.update", Config.AUTH, params));
    }


    @Data
    public static class ArgItemDiscoverySwitchStatus {
        ArrayList<Long> itemids;
        @NotNull(message = "状态不能为空")
        @Min(value = 0, message = "0为开启;1为关闭")
        @Max(value = 1, message = "0为开启;1为关闭")
        Integer status;
    }

    @Override
    public Result<JSONObject> itemDiscoverySwitchStatus(ArgItemDiscoverySwitchStatus args) {
        UpdateWrapper<Items> uw = new UpdateWrapper<>();
        uw.set("status", args.status);
        uw.in("itemid", args.itemids);
        itemsMapper.update(null, uw);
        return Result.success(null);
    }


    @Data
    public static class ArgGetItemprototype {
        Integer pageNum;
        Integer pageSize;
        ArrayList<Long> hostids;
        ArrayList<Long> discoveryids;
        Boolean selectMasterItem;
        Boolean selectTemplate;
        Boolean selectTags;
    }

    @Override
    public Result<JSONObject> getItemprototype(ArgGetItemprototype args) {
        QueryWrapper<Items> qw = new QueryWrapper<>();
        qw.eq("flags", Config.host.ZBX_FLAG_DISCOVERY_PROTOTYPE);

        if (null != args.getHostids() && args.getHostids().size() > 0) {
            qw.in("hostid", args.getHostids());
        }
        if (null != args.discoveryids && args.discoveryids.size() > 0) {
            qw.inSql("itemid", "SELECT DISTINCT itemid from item_discovery where parent_itemid in (" + Common.getArrayList(args.discoveryids) + ")");
        }
        Integer count = itemsMapper.selectCount(qw);

        if (count == 0) {
            return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT, new ArrayList<>(), count));
        }

        ArrayList<Long> itemids = new ArrayList<>();
        qw.select("itemid");
        qw.orderByAsc("name_upper");
        ServiceUtil.Page page = ServiceUtil.init_page(args.pageNum, args.pageSize,count);
        qw.last("LIMIT " + (page.getNum() - 1) * page.getSize() + ", " + page.getSize());
        List<Items> items = itemsMapper.selectList(qw);
        for (Items item : items) {
            itemids.add(item.getItemid());
        }

        List<ResItemList> result = itemsMapper.itemList(new QueryWrapper<Items>().in("i.itemid", itemids).orderByAsc("i.name_upper"));

        // master_item
        if (null != args.getSelectMasterItem() && args.getSelectMasterItem()) {
            ArrayList<Long> master_itemids = new ArrayList<>();
            for (ResItemList r : result) {
                if (null != r.getMaster_itemid()) {
                    master_itemids.add(r.getMaster_itemid());
                }
            }
            if (master_itemids.size() > 0) {
                List<Items> master_items = itemsMapper.selectList(
                        new QueryWrapper<Items>()
                                .select("itemid,name,flags")
                                .in("flags", Arrays.asList(Config.host.ZBX_FLAG_DISCOVERY_NORMAL, Config.host.ZBX_FLAG_DISCOVERY_CREATED, Config.host.ZBX_FLAG_DISCOVERY_PROTOTYPE))
//                                .eq("flags", Config.host.ZBX_FLAG_DISCOVERY_PROTOTYPE)
                                .in("itemid", master_itemids));
                for (ResItemList r : result) {
                    for (Items i : master_items) {
                        if (i.getItemid().equals(r.getMaster_itemid())) {
                            r.setMaster_item(i.getName());
                            // 监控项原型的依赖可以为监控项或是另一个监控项原型
                            if (i.getFlags() == Config.host.ZBX_FLAG_DISCOVERY_PROTOTYPE) {
                                r.setMaster_item_flags(i.getFlags());
                            }
                            break;
                        }
                    }
                }
            }
        }

        // item_template
        if (null != args.getSelectTemplate() && args.getSelectTemplate()) {
            ArrayList<Long> template_itemids = new ArrayList<>();
            for (ResItemList r : result) {
                if (null != r.getTemplateid()) {
                    template_itemids.add(r.getTemplateid());
                }
            }
            if (template_itemids.size() > 0) {
                List<Items> template_items = itemsMapper.selectList(
                        new QueryWrapper<Items>()
                                .select("itemid,hostid")
                                .eq("flags", Config.host.ZBX_FLAG_DISCOVERY_PROTOTYPE)
                                .in("itemid", template_itemids));
                ArrayList<Long> template_hostids = new ArrayList<>();
                for (Items r : template_items) {
                    if (null != r.getHostid()) {
                        template_hostids.add(r.getHostid());
                    }
                }
                if (template_hostids.size() > 0) {
                    List<Hosts> template_hosts = hostsMapper.selectList(
                            new QueryWrapper<Hosts>()
                                    .select("hostid,name")
                                    .eq("status", Config.host.HOST_STATUS_TEMPLATE)
                                    .in("hostid", template_hostids));
                    for (ResItemList r : result) {
                        for (Items ti : template_items) {
                            if (ti.getItemid().equals(r.getTemplateid())) {
                                for (Hosts h : template_hosts) {
                                    if (h.getHostid().equals(ti.getHostid())) {
                                        r.setTemplate(h.getName());
                                        break;
                                    }
                                }
                                break;
                            }
                        }
                    }
                }
            }
        }

        // 标签
        if (null != args.getSelectTags() && args.getSelectTags()) {
            List<ItemTag> itemTags = itemTagMapper.selectList(new QueryWrapper<ItemTag>().in("itemid", itemids));
            for (ResItemList r : result) {
                r.setTags(itemTags.stream()
                        .filter(i -> i.getItemid().equals(r.getItemid()))
                        .collect(Collectors.toList()));
            }
        }

        return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT, result, count, page.getNum(), page.getSize()));
    }

    @Data
    public static class ArgItemprototypeDetail {
        Long itemid;
    }

    @Override
    public Result<JSONObject> itemprototypeDetail(ArgItemprototypeDetail args) {
        Long itemid = args.itemid;
        QueryWrapper<Items> qw = new QueryWrapper<>();
        qw.eq("flags", Config.host.ZBX_FLAG_DISCOVERY_PROTOTYPE);
        qw.eq("itemid", itemid);
        Integer count = itemsMapper.selectCount(qw);
        if (!count.equals(1)) {
            return Result.fail(HttpStatusEnum.BAD_REQUEST, "无效id");
        }
        Items item = itemsMapper.selectOne(qw);
        if (null != item.getMaster_itemid()) {
            Items master_item = itemsMapper.selectOne(
                    new QueryWrapper<Items>()
                            .select("itemid,name,flags")
                            .in("flags", Arrays.asList(Config.host.ZBX_FLAG_DISCOVERY_NORMAL, Config.host.ZBX_FLAG_DISCOVERY_CREATED, Config.host.ZBX_FLAG_DISCOVERY_PROTOTYPE))
//                                .eq("flags", Config.host.ZBX_FLAG_DISCOVERY_PROTOTYPE)
                            .eq("itemid", item.getMaster_itemid()));
            item.setMaster_item(master_item.getName());
            // 监控项原型的依赖可以为监控项或是另一个监控项原型
            if (master_item.getFlags() == Config.host.ZBX_FLAG_DISCOVERY_PROTOTYPE) {
                item.setMaster_item_flags(master_item.getFlags());
            }
        }
        if (null != item.getTemplateid()) {
            Items template_item = itemsMapper.selectOne(
                    new QueryWrapper<Items>()
                            .select("itemid,hostid")
                            .eq("flags", Config.host.ZBX_FLAG_DISCOVERY_PROTOTYPE)
                            .eq("itemid", item.getTemplateid()));
            Hosts template_host = hostsMapper.selectOne(
                    new QueryWrapper<Hosts>()
                            .select("hostid,name")
                            .eq("status", Config.host.HOST_STATUS_TEMPLATE)
                            .eq("hostid", template_item.getHostid()));
            item.setTemplate(template_host.getName());
        }
        List<ItemTag> tags = itemTagMapper.selectList(new QueryWrapper<ItemTag>().eq("itemid", itemid));
        List<ItemPreproc> preprocs = itemPreprocMapper.selectList(new QueryWrapper<ItemPreproc>().eq("itemid", itemid));
        item.setTags(tags);
        item.setPreprocessing(preprocs);
        return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT, item));
    }

    @Data
    public static class ArgItemprototypeCreate {
        @NotNull(message = "模板id不能为空")
        Long hostid;
        @NotNull(message = "自动发现规则id不能为空")
        Long ruleid;
        @NotNull(message = "名称不能为空")
        String name;
        @NotNull(message = "键值不能为空")
        String key_;
        @NotNull(message = "类型不能为空")
        Integer type;    // 0 - agent监控项; 18 - 依赖监控项; 21 - 脚本监控项
        @NotNull(message = "信息类型不能为空")
        Integer value_type;
        Long master_itemid;
        String units;
        @NotNull(message = "更新间隔不能为空")
        String delay;
        ArrayList<ArgTag> tags;
        ArrayList<ArgPreproc> preprocessing;
        String history;
        String trends;
        String description;
        Integer valuemapid;
    }

    @Override
    public Result<JSONObject> itemprototypeCreate(ArgItemprototypeCreate args) {
        JSONObject params = new JSONObject();
        params.put("hostid", args.hostid);
        params.put("ruleid", args.ruleid);
        params.put("name", args.name);
        params.put("key_", args.key_);
        params.put("type", args.type);
        params.put("value_type", args.value_type);
        params.put("delay", args.delay);
        if (args.type.equals(Config.item.ITEM_TYPE_DEPENDENT)) {
            if (null == args.master_itemid) {
                return Result.fail(HttpStatusEnum.BAD_REQUEST, "缺少依赖item信息");
            }
            params.put("delay", 0);
            params.put("master_itemid", args.master_itemid);
        }
        params.put("units", args.units);
        params.put("tags", args.tags);
        params.put("preprocessing", args.preprocessing);
        params.put("history", args.history);
        params.put("trends", args.trends);
        params.put("description", args.description);
        params.put("valuemapid", args.valuemapid);

        return Result.success(ZabbixApi.post("itemprototype.create", Config.AUTH, params));
    }

    @Data
    public static class ArgItemprototypeDelete {
        ArrayList<Long> itemids;
    }

    @Override
    public Result<JSONObject> itemprototypeDelete(ArgItemprototypeDelete args) {
        return Result.success(ZabbixApi.post("itemprototype.delete", Config.AUTH, args.itemids));
    }

    @Data
    public static class ArgItemprototypeUpdate {
        @NotNull(message = "监控项原型id不能为空")
        Long itemid;
        String name;
        String key_;
        Integer type;    // 0 - agent监控项; 18 - 依赖监控项; 21 - 脚本监控项
        Integer value_type;
        Long master_itemid;
        String units;
        String delay;
        ArrayList<ArgTag> tags;
        ArrayList<ArgPreproc> preprocessing;
        String history;
        String trends;
        String description;
        Integer valuemapid;
    }

    @Override
    public Result<JSONObject> itemprototypeUpdate(ArgItemprototypeUpdate args) {
        JSONObject params = new JSONObject();
        params.put("itemid", args.itemid);
        params.put("name", args.name);
        params.put("key_", args.key_);
        params.put("type", args.type);
        params.put("value_type", args.value_type);
        params.put("delay", args.delay);
        if (args.type.equals(Config.item.ITEM_TYPE_DEPENDENT)) {
            if (null == args.master_itemid) {
                return Result.fail(HttpStatusEnum.BAD_REQUEST, "缺少依赖item信息");
            }
            params.put("delay", 0);
            params.put("master_itemid", args.master_itemid);
        }
        params.put("units", args.units);
        params.put("tags", args.tags);
        params.put("preprocessing", args.preprocessing);
        params.put("history", args.history);
        params.put("trends", args.trends);
        params.put("description", args.description);
        params.put("valuemapid", args.valuemapid);

        return Result.success(ZabbixApi.post("itemprototype.update", Config.AUTH, params));
    }

    @Data
    public static class ArgItemprototypesSwitchStatus {
        ArrayList<Long> itemids;
        @NotNull(message = "状态不能为空")
        @Min(value = 0, message = "0为开启;1为关闭")
        @Max(value = 1, message = "0为开启;1为关闭")
        Integer status;
    }

    @Override
    public Result<JSONObject> itemprototypesSwitchStatus(ArgItemprototypesSwitchStatus args) {
        UpdateWrapper<Items> uw = new UpdateWrapper<>();
        uw.set("status", args.status);
        uw.in("itemid", args.itemids);
        itemsMapper.update(null, uw);
        return Result.success(null);
    }


    @Data
    public static class ArgAllItems {
        Long hostid;
    }

    // 主机监控项主要信息
    @Override
    public Result<JSONObject> allItems(ArgAllItems args) {
        Integer count = hostsMapper.selectCount(new QueryWrapper<Hosts>().eq("status", Config.host.HOST_STATUS_TEMPLATE).eq("hostid", args.hostid));
        if (count != 1) {
            return Result.fail(HttpStatusEnum.BAD_REQUEST, "无效id");
        }
        List<ResItemList> result = itemsMapper.itemList(
                new QueryWrapper<Items>()
                        .eq("i.hostid", args.hostid)
                        .eq("i.flags", Config.host.ZBX_FLAG_DISCOVERY_NORMAL));
        return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT, result));
    }

    @Data
    public static class ArgAllItemprototype {
        Long itemid;
    }

    // 全部监控项原型
    @Override
    public Result<JSONObject> allItemprototype(ArgAllItemprototype args) {
        Integer count = itemsMapper.selectCount(new QueryWrapper<Items>().eq("flags", Config.host.ZBX_FLAG_DISCOVERY_RULE).eq("itemid", args.itemid));
        if (count != 1) {
            return Result.fail(HttpStatusEnum.BAD_REQUEST, "无效id");
        }
        List<Items> result = itemsMapper.selectList(
                new QueryWrapper<Items>()
                        .inSql("itemid", "SELECT DISTINCT itemid FROM item_discovery WHERE parent_itemid =" + args.itemid)
                        .eq("flags", Config.host.ZBX_FLAG_DISCOVERY_PROTOTYPE));
        return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT, result));
    }

    @Data
    public static class ArgTagList {
        ArrayList<Long> hostids;
    }


    @Override
    public Result<JSONObject> tagList(ArgTagList args) {
        return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT, itemTagMapper.tagList(args.hostids)));
    }

    @Data
    public static class ArgHistory {
        Integer pageNum;
        Integer pageSize;
        Long itemid;
        String time_from;
        String time_till;
    }

    @Override
    public Result<JSONObject> history(ArgHistory args) {
        Long itemid = args.itemid;
        QueryWrapper<Items> qw = new QueryWrapper<>();
        qw.inSql("hostid", "SELECT hostid from hosts where status <> " + Config.host.HOST_STATUS_TEMPLATE);
        qw.in("flags", Arrays.asList(Config.host.ZBX_FLAG_DISCOVERY_NORMAL, Config.host.ZBX_FLAG_DISCOVERY_CREATED));
        qw.eq("itemid", itemid);
        Integer item_count = itemsMapper.selectCount(qw);
        if (!item_count.equals(1)) {
            return Result.fail(HttpStatusEnum.BAD_REQUEST, "无效id");
        }

        Items items = itemsMapper.selectOne(new QueryWrapper<Items>().eq("itemid", itemid));



        QueryWrapper his_qw = new QueryWrapper<>();
        his_qw.eq("itemid", itemid);

        String time_from = args.time_from;
        his_qw.ge((null != time_from && "" != time_from), "clock", time_from);

        String time_till = args.time_till;
        his_qw.le((null != time_till && "" != time_till), "clock", time_till);

        Integer count = 0;
        switch (items.getValue_type()) {
            case Config.item.ITEM_VALUE_TYPE_FLOAT:
                count = historyMapper.count_history(his_qw);
                break;
            case Config.item.ITEM_VALUE_TYPE_STR:
                count = historyMapper.count_history_str(his_qw);
                break;
            case Config.item.ITEM_VALUE_TYPE_LOG:
                count = historyMapper.count_history_log(his_qw);
                break;
            case Config.item.ITEM_VALUE_TYPE_UINT64:
                count = historyMapper.count_history_uint(his_qw);
                break;
            case Config.item.ITEM_VALUE_TYPE_TEXT:
                count = historyMapper.count_history_text(his_qw);
                break;
        }

        ServiceUtil.Page page = ServiceUtil.init_page(args.pageNum, args.pageSize,count);
        his_qw.orderByDesc("clock", "ns");
        his_qw.last("LIMIT " + (page.getNum() - 1) * page.getSize() + ", " + page.getSize());

        ArrayList<ResHistory> history = null;
        switch (items.getValue_type()) {
            case Config.item.ITEM_VALUE_TYPE_FLOAT:
                history = historyMapper.history(his_qw);
                break;
            case Config.item.ITEM_VALUE_TYPE_STR:
                history = historyMapper.history_str(his_qw);
                break;
            case Config.item.ITEM_VALUE_TYPE_LOG:
                history = historyMapper.history_log(his_qw);
                break;
            case Config.item.ITEM_VALUE_TYPE_UINT64:
                history = historyMapper.history_uint(his_qw);
                break;
            case Config.item.ITEM_VALUE_TYPE_TEXT:
                history = historyMapper.history_text(his_qw);
                break;
        }

        return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT, history, count, page.getNum(), page.getSize()));
    }

}
