package com.zh.test_zabbix.serviceImpl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zh.test_zabbix.config.Config;
import com.zh.test_zabbix.mapper.HistoryMapper;
import com.zh.test_zabbix.mapper.MsCustomItemsMapper;
import com.zh.test_zabbix.mapper.res.ResCustomItems;
import com.zh.test_zabbix.mapper.res.ResHistory;
import com.zh.test_zabbix.model.MsCustomItems;
import com.zh.test_zabbix.service.IMsCustomItemsService;
import com.zh.test_zabbix.serviceImpl.util.ServiceUtil;
import com.zh.test_zabbix.utils.result.HttpStatusEnum;
import com.zh.test_zabbix.utils.result.Result;
import lombok.Data;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.validation.constraints.NotNull;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author zh
 * @since 2023-11-14
 */
@Service
public class MsCustomItemsServiceImpl extends ServiceImpl<MsCustomItemsMapper, MsCustomItems> implements IMsCustomItemsService {

    @Resource
    private MsCustomItemsMapper customItemsMapper;

    @Resource
    private HistoryMapper historyMapper;

    @Data
    public static class argInfo {
        @NotNull(message = "主机id不能为空")
        Long hostid;
        @NotNull(message = "类型不能为空")
        Integer type_;
    }

    @Override
    public Result<JSONObject> info(argInfo args) {
        QueryWrapper<MsCustomItems> qw = new QueryWrapper<>();
        qw.eq("i.hostid", args.hostid);
        qw.eq("ci.type", args.type_);
        qw.eq("ci.status", Config.CUSTOM_ITEM_STATUS_ENABLE);
        qw.in("flags", Arrays.asList(Config.host.ZBX_FLAG_DISCOVERY_NORMAL, Config.host.ZBX_FLAG_DISCOVERY_CREATED));
        List<ResCustomItems> result = customItemsMapper.list(qw);
        if (result.size() == 0) {
            return Result.fail(HttpStatusEnum.BAD_REQUEST, "无查询结果");
        }
        HashMap<Integer, ArrayList<Long>> map = new HashMap<>();
        for (ResCustomItems r : result) {
            if (r.getHistory().equals(Config.CUSTOM_ITEM_HISTORY_LATEST)) {
                Integer value_type = r.getValue_type();
                if (map.get(value_type) == null) {
                    map.put(value_type, new ArrayList());
                }
                map.get(value_type).add(r.getItemid());
            } else if (r.getHistory().equals(Config.CUSTOM_ITEM_HISTORY_PERIOD)) {
                QueryWrapper his_qw;
                ArrayList<ResHistory> history = null;
                his_qw = new QueryWrapper<>();
                his_qw.eq("itemid", r.getItemid());
                his_qw.orderByDesc("clock", "ns");
                Long time_from = System.currentTimeMillis() / 1000 - 60 * 60;
                his_qw.gt("clock", time_from);
                switch (r.getValue_type()) {
                    case Config.item.ITEM_VALUE_TYPE_FLOAT:
                        history = historyMapper.history(his_qw);
                        break;
                    case Config.item.ITEM_VALUE_TYPE_STR:
                        history = historyMapper.history_str(his_qw);
                        break;
                    case Config.item.ITEM_VALUE_TYPE_LOG:
                        history = historyMapper.history_log(his_qw);
                        break;
                    case Config.item.ITEM_VALUE_TYPE_UINT64:
                        history = historyMapper.history_uint(his_qw);
                        break;
                    case Config.item.ITEM_VALUE_TYPE_TEXT:
                        history = historyMapper.history_text(his_qw);
                        break;
                }
                r.setLast_value(history);
            }
        }

        QueryWrapper his_qw;
        for (Map.Entry<Integer, ArrayList<Long>> e : map.entrySet()) {
            ArrayList<ResHistory> history = null;
            his_qw = new QueryWrapper<>();
            his_qw.in("itemid", e.getValue());
            switch (e.getKey()) {
                case Config.item.ITEM_VALUE_TYPE_FLOAT:
                    history = historyMapper.history_list(his_qw);
                    break;
                case Config.item.ITEM_VALUE_TYPE_STR:
                    history = historyMapper.history_str_list(his_qw);
                    break;
                case Config.item.ITEM_VALUE_TYPE_LOG:
                    history = historyMapper.history_log_list(his_qw);
                    break;
                case Config.item.ITEM_VALUE_TYPE_UINT64:
                    history = historyMapper.history_uint_list(his_qw);
                    break;
                case Config.item.ITEM_VALUE_TYPE_TEXT:
                    history = historyMapper.history_text_list(his_qw);
                    break;
            }
            for (ResCustomItems r : result) {
                if (r.getHistory().equals(Config.CUSTOM_ITEM_HISTORY_LATEST) && r.getValue_type().equals(e.getKey())) {
                    for (ResHistory h : history) {
                        if (h.getItemid().equals(r.getItemid())) {
                            r.setLast_value(h.getValue());
                            break;
                        }
                    }
                }
            }
        }

        List<MsCustomItems> res = customItemsMapper.selectList(new QueryWrapper<MsCustomItems>().eq("type", args.type_));
        HashMap<String, Object> resMap = new HashMap<>();
        for (MsCustomItems r : res) {
            if (r.getKind() == Config.CUSTOM_ITEM_KIND_NORMAL) {
                for (ResCustomItems r1 : result) {
                    if (r1.getId().equals(r.getId())) {
                        resMap.put(r.getName(), r1.getLast_value());
                        break;
                    }
                }
            } else if (r.getKind() == Config.CUSTOM_ITEM_KIND_WITHARGS) {
                List<ResCustomItems> collect = result.stream()
                        .filter(i -> i.getId().equals(r.getId()))
                        .collect(Collectors.toList());
                HashMap<String, Object> collMap = new HashMap<>();
                for (ResCustomItems c : collect) {
                    collMap.put(split_key(c.getKey_()), c.getLast_value());
                }
                resMap.put(r.getName(), collMap);
            }
        }

        return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT, resMap));
    }

    static String split_key(String key) {
        int i = key.indexOf("[");
        int e = key.indexOf("]");
        if (i == -1 || e == -1) {
            return key;
        }
        return key.substring(i + 1, e);
    }

}
