package com.zh.test_zabbix.serviceImpl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zh.test_zabbix.config.Config;
import com.zh.test_zabbix.mapper.*;
import com.zh.test_zabbix.mapper.res.ResProblem;
import com.zh.test_zabbix.model.*;
import com.zh.test_zabbix.service.IProblemService;
import com.zh.test_zabbix.serviceImpl.util.ServiceUtil;
import com.zh.test_zabbix.utils.Common;
import com.zh.test_zabbix.utils.result.Result;
import lombok.Data;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author zh
 * @since 2023-11-14
 */
@Service
public class ProblemServiceImpl extends ServiceImpl<ProblemMapper,Problem> implements IProblemService {

    @Resource
    private HostsMapper hostsMapper;

    @Resource
    private MsHostCollectionMapper collectionMapper;

    @Resource
    private TriggersMapper triggersMapper;

    @Resource
    private TriggerDependsMapper triggerDependsMapper;

    @Resource
    private ProblemTagMapper problemTagMapper;

    @Resource
    private ProblemMapper problemMapper;

    @Data
    public static class ArgCount {
        ArrayList<Long> collectionids;
        ArrayList<Long> groupids;
        ArrayList<Long> hostids;
        ArrayList<Long> itemids;
        ArrayList<Long> triggerids;

        ArrayList<Integer> severity;
        String name;

        String time_from;
        String time_till;

    }

    @Override
    public Result<JSONObject> countBySeverity(ArgCount args) {
        QueryWrapper<Problem> qw = new QueryWrapper<>();

        qw.eq("p.source", Config.Event.EVENT_SOURCE_TRIGGERS);
        qw.eq("p.object", Config.Event.EVENT_OBJECT_TRIGGER);
        qw.isNull("p.cause_eventid");
        qw.isNull("p.r_eventid");

        ArrayList<Long> arg_collectionids = args.collectionids;
        if (null != arg_collectionids && arg_collectionids.size() > 0) {
            qw.in("h.collectionid", arg_collectionids);
        }
        ArrayList<Long> groupids = args.groupids;
        if (null != groupids && groupids.size() > 0) {
            qw.inSql("h.hostid", "SELECT hostid FROM hosts_groups WHERE groupid in (" + Common.getArrayList(groupids) + ")");
        }
        ArrayList<Long> hostids = args.hostids;
        if (null != hostids && hostids.size() > 0) {
            qw.in("h.hostid", hostids);
        }
        ArrayList<Long> itemids = args.itemids;
        if (null != itemids && itemids.size() > 0) {
            qw.in("i.itemid", itemids);
        }
        ArrayList<Long> triggerids = args.triggerids;
        if (null != triggerids && triggerids.size() > 0) {
            qw.in("p.objectid", triggerids);
        }
        ArrayList<Integer> severity = args.severity;
        if (null != severity) {
            qw.in("p.severity", severity);
        }
        String name = args.name;
        if (null != name && ""!=name) {
            qw.like("upper(p.name)", name.toUpperCase());
        }
        String time_from = args.time_from;
        if (null != time_from && "" != time_from) {
            qw.gt("p.clock", time_from);
        }
        String time_till = args.time_till;
        if (null != time_till && "" != time_till) {
            qw.lt("p.clock", time_till);
        }

        return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT,  problemMapper.countBySeverity(qw)));
    }

    @Data
    public static class ArgGet {
        Integer pageNum;
        Integer pageSize;

        ArrayList<Long> collectionids;
        ArrayList<Long> groupids;
        ArrayList<Long> hostids;
        ArrayList<Long> itemids;
        ArrayList<Long> triggerids;

        ArrayList<Integer> severity;
        String name;

        String time_from;
        String time_till;

        Boolean selectcollection;
        Boolean selectTrigger;
        Boolean selectTags;
    }

    @Override
    public Result get(ArgGet args) {
        QueryWrapper<Problem> qw = new QueryWrapper<>();
        qw.eq("p.source", Config.Event.EVENT_SOURCE_TRIGGERS);
        qw.eq("p.object", Config.Event.EVENT_OBJECT_TRIGGER);
        qw.isNull("p.cause_eventid");
        qw.isNull("p.r_eventid");

        ArrayList<Long> arg_collectionids = args.collectionids;
        if (null != arg_collectionids && arg_collectionids.size() > 0) {
            qw.in("h.collectionid", arg_collectionids);
        }
        ArrayList<Long> groupids = args.groupids;
        if (null != groupids && groupids.size() > 0) {
            qw.inSql("h.hostid", "SELECT hostid FROM hosts_groups WHERE groupid in (" + Common.getArrayList(groupids) + ")");
        }
        ArrayList<Long> hostids = args.hostids;
        if (null != hostids && hostids.size() > 0) {
            qw.in("h.hostid", hostids);
        }
        ArrayList<Long> itemids = args.itemids;
        if (null != itemids && itemids.size() > 0) {
            qw.in("i.itemid", itemids);
        }
        ArrayList<Long> triggerids = args.triggerids;
        if (null != triggerids && triggerids.size() > 0) {
            qw.in("p.objectid", triggerids);
        }
        ArrayList<Integer> severity = args.severity;
        if (null != severity) {
            qw.in("p.severity", severity);
        }
       String name = args.name;
        if (null != name && ""!=name) {
            qw.like("upper(p.name)", name.toUpperCase());
        }
        String time_from = args.time_from;
        if (null != time_from && "" != time_from) {
            qw.gt("p.clock", time_from);
        }
        String time_till = args.time_till;
        if (null != time_till && "" != time_till) {
            qw.lt("p.clock", time_till);
        }

        Integer count = problemMapper.count(qw);
        if (count == 0) {
            return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT, new ArrayList<>(), count));
        }

        qw.orderByDesc("p.eventid");
        ServiceUtil.Page page = ServiceUtil.init_page(args.pageNum, args.pageSize,count);
        qw.last("LIMIT " + (page.getNum() - 1) * page.getSize() + ", " + page.getSize());
        List<ResProblem> result = problemMapper.get(qw);
        if (result.size() == 0) {
            return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT, new ArrayList<>(), count));
        }
        ArrayList<Long> eventids = new ArrayList<>();
        ArrayList<Long> objectids = new ArrayList<>();
        ArrayList<Long> collectionids = new ArrayList<>();

        for (ResProblem r : result) {
            eventids.add(r.getEventid());
            objectids.add(r.getTriggerid());
            collectionids.add(r.getCollectionid());
        }

        if(null!= args.selectcollection && args.selectcollection){
            if(collectionids.size()>0) {
                List<MsHostCollection> collections = collectionMapper.selectList(new QueryWrapper<MsHostCollection>().in("id", collectionids));
                for (ResProblem r : result) {
                    for (MsHostCollection coll : collections) {
                        if (coll.getId().equals(r.getCollectionid())) {
                            r.setHc_name(coll.getName());
                            break;
                        }
                    }
                }
            }
        }

        if (null != args.selectTrigger && args.selectTrigger) {
            List<Triggers> triggers = triggersMapper.selectList(new QueryWrapper<Triggers>().in("triggerid", objectids));
            if (null != triggers) {
                for (ResProblem r : result) {
                    for (Triggers t : triggers) {
                        if (t.getTriggerid().equals(r.getTriggerid())) {
                            r.setTrigger_expression(t.getExpression());
                            r.setTrigger_comment(t.getComments());
                            break;
                        }
                    }
                }
            }

            List<TriggerDepends> triggerups = triggerDependsMapper.selectList(new QueryWrapper<TriggerDepends>().in("triggerid_up", objectids));
            if (null != triggerups && triggerups.size()>0) {
                ArrayList<Long> triggerupids = new ArrayList<>();
                for (TriggerDepends tp : triggerups) {
                    triggerupids.add(tp.getTriggerid_down());
                }
                List<Triggers> triggers1 = triggersMapper.selectList(new QueryWrapper<Triggers>().in("triggerid", triggerupids));
                if (null != triggers1&& triggers1.size()>0) {
                    for (ResProblem r : result) {
                        for (TriggerDepends tu : triggerups) {
                            if (tu.getTriggerid_up().equals(r.getTriggerid())) {
                                for (Triggers t : triggers1) {
                                    if (t.getTriggerid().equals(tu.getTriggerid_down())) {
                                        r.setTriggerup_description(t.getDescription());
                                        break;
                                    }
                                }
                                break;
                            }
                        }
                    }
                }
            }
            List<TriggerDepends> triggerdowns = triggerDependsMapper.selectList(new QueryWrapper<TriggerDepends>().in("triggerid_down", objectids));
            if (null != triggerdowns&& triggerdowns.size()>0) {
                ArrayList<Long> triggerdownids = new ArrayList<>();
                for (TriggerDepends td : triggerdowns) {
                    triggerdownids.add(td.getTriggerid_up());
                }
                List<Triggers> triggers2 = triggersMapper.selectList(new QueryWrapper<Triggers>().in("triggerid", triggerdownids));
                if (null != triggers2&& triggers2.size()>0) {
                    for (ResProblem r : result) {
                        for (TriggerDepends td : triggerdowns) {
                            if (td.getTriggerid_down().equals(r.getTriggerid())) {
                                for (Triggers t : triggers2) {
                                    if (t.getTriggerid().equals(td.getTriggerid_up())) {
                                        r.setTriggerup_description(t.getDescription());
                                        break;
                                    }
                                }
                                break;
                            }
                        }
                    }
                }
            }
        }

        if (null != args.selectTags && args.selectTags) {
            List<ProblemTag> tags = problemTagMapper.selectList(new QueryWrapper<ProblemTag>().in("eventid", eventids));
            for (ResProblem r : result) {
                List<ProblemTag> collect = tags.stream()
                        .filter(i -> i.getEventid().equals(r.getEventid()))
                        .collect(Collectors.toList());
                r.setTags(collect);
            }
        }

        return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT, result, count, page.getNum(),page.getSize()));
    }
}
