package com.zh.test_zabbix.serviceImpl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zh.test_zabbix.config.Config;
import com.zh.test_zabbix.mapper.RoleMapper;
import com.zh.test_zabbix.mapper.UsersMapper;
import com.zh.test_zabbix.mapper.res.ResUsers;
import com.zh.test_zabbix.model.Role;
import com.zh.test_zabbix.model.Users;
import com.zh.test_zabbix.service.IRoleService;
import com.zh.test_zabbix.serviceImpl.util.ServiceUtil;
import com.zh.test_zabbix.utils.Valid;
import com.zh.test_zabbix.utils.result.Result;
import lombok.Data;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author zh
 * @since 2023-11-22
 */
@Service
public class RoleServiceImpl extends ServiceImpl<RoleMapper, Role> implements IRoleService {

    @Resource
    private RoleMapper roleMapper;

    @Resource
    private UsersMapper usersMapper;

    @Data
    public static class ArgGet {
        Integer pageNum;
        Integer pageSize;
        String name;
        Boolean selectUsers;
    }

    @Override
    public Result<JSONObject> get(ArgGet args) {
        // 是否page格式
        Boolean isPage = true;
        if (null != args.pageSize && args.pageSize == 0) {
            isPage = false;
        }

        QueryWrapper<Role> qw = new QueryWrapper<>();
        qw.like(Valid.isNotNull(args.name), "UPPER(name)", (null == args.name ? "" : args.name).toUpperCase());

        Integer count = roleMapper.selectCount(qw);
        if (count == 0) {
            return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT, new ArrayList<>(), count));
        }

        ServiceUtil.Page page = new ServiceUtil.Page();
        if (isPage) {
            page = ServiceUtil.init_page(args.pageNum, args.pageSize,count);
            qw.last("LIMIT " + (page.getNum() - 1) * page.getSize() + ", " + page.getSize());
        }
        qw.orderByAsc("UPPER(name)");

        List<Role> result = roleMapper.selectList(qw);

        ArrayList<Long> roleids = new ArrayList<>();
        for (Role r : result) {
            roleids.add(r.getRoleid());
        }

        if (Valid.isTrue(args.selectUsers)) {
            List<ResUsers> users = usersMapper.get(new QueryWrapper<Users>()
                    .in("roleid", roleids));
            if (null != users) {
                ArrayList<Long> userids = new ArrayList<>();
                for (ResUsers u : users) {
                    userids.add(u.getUserid());
                }
                List<Map> user_status = usersMapper.user_status(new QueryWrapper<Users>().in("ug.userid", userids));
                for (ResUsers u : users) {
                    for (Map us : user_status) {
                        if (Long.valueOf(us.get("userid").toString()).equals(u.getUserid())) {
                            u.setUsers_status(Integer.parseInt(us.get("users_status").toString()));
                            break;
                        }
                    }
                }
                for (Role r : result) {
                    List<ResUsers> collect = users.stream()
                            .filter(i -> i.getRoleid().equals(r.getRoleid()))
                            .collect(Collectors.toList());
                    r.setUsers(collect);
                }
            }
        }

        if (isPage) {
            return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT, result, count, page.getNum(), page.getSize()));
        } else {
            return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT, result));
        }
    }

}
