package com.zh.test_zabbix.serviceImpl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zh.test_zabbix.config.Config;
import com.zh.test_zabbix.mapper.*;
import com.zh.test_zabbix.mapper.res.ResItemList;
import com.zh.test_zabbix.model.*;
import com.zh.test_zabbix.service.ITriggersService;
import com.zh.test_zabbix.serviceImpl.util.ServiceUtil;
import com.zh.test_zabbix.utils.ZabbixApi;
import com.zh.test_zabbix.utils.result.HttpStatusEnum;
import com.zh.test_zabbix.utils.result.Result;
import lombok.Data;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author zh
 * @since 2023-10-26
 */
@Service
public class TriggersServiceImpl extends ServiceImpl<TriggersMapper, Triggers> implements ITriggersService {

    @Resource
    private TriggersMapper triggersMapper;

    @Resource
    private TriggerTagMapper triggerTagMapper;

    @Resource
    private HostsMapper hostsMapper;

    @Resource
    private ItemsMapper itemsMapper;

    @Resource
    private FunctionsMapper functionsMapper;


    @Data
    public static class ArgGet {
        Integer pageNum;
        Integer pageSize;
        ArrayList<Long> hostids;
        ArrayList<Long> itemids;
        String name;
        Boolean selectTags;

        Integer kind; // 1为模板，否则为主机
    }

    @Override
    public Result<JSONObject> get(ArgGet args) {

        // 主机和模板参数不同
        Boolean isTemplate = (null != args.kind && args.kind.equals(1)) ? true : false;

        QueryWrapper<Triggers> qw = new QueryWrapper<>();

        qw.in("t.flags", Arrays.asList(Config.host.ZBX_FLAG_DISCOVERY_NORMAL, Config.host.ZBX_FLAG_DISCOVERY_CREATED));
        if (isTemplate) {
            qw.eq("h.status", Config.host.HOST_STATUS_TEMPLATE);
        } else {
            qw.ne("h.status", Config.host.HOST_STATUS_TEMPLATE);
        }

        ArrayList<Long> arg_hostids = args.getHostids();
        if (null != arg_hostids && arg_hostids.size() > 0) {
            qw.in("h.hostid", arg_hostids);
        }
        ArrayList<Long> arg_itemids = args.getItemids();
        if (null != arg_itemids && arg_itemids.size() > 0) {
            qw.in("i.itemid", arg_itemids);
        }
        String arg_name = args.getName();
        if (null != arg_name && "" != arg_name) {
            String arg_name_upper = arg_name.toUpperCase();
            qw.like("UPPER(t.description)", arg_name_upper);
        }

        Integer count = triggersMapper.count(qw);

        ArrayList<Long> triggerids = new ArrayList<>();
        ServiceUtil.Page page = ServiceUtil.init_page(args.pageNum, args.pageSize,count);
        qw.orderByAsc("UPPER(t.description)");
        qw.last("LIMIT " + (page.getNum() - 1) * page.getSize() + ", " + page.getSize());
        List<Triggers> triggeridsList = triggersMapper.triggerids(qw);
        for (Triggers t : triggeridsList) {
            triggerids.add(t.getTriggerid());
        }
        if (triggerids.size() == 0) {
            return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT, new ArrayList<>()));
        }

        List<Triggers> result = triggersMapper.selectList(new QueryWrapper<Triggers>().in("triggerid", triggerids).orderByAsc("UPPER(description)"));

        //  处理expression部分的macro
        ArrayList<Integer> functionids = new ArrayList<>();
        for (Triggers r : result) {
            Pattern pattern = Pattern.compile("\\{(\\d+)\\}");
            Matcher matcher = pattern.matcher(r.getExpression());
            while (matcher.find()) {
                Integer number = Integer.valueOf(matcher.group().replace("{", "").replace("}", ""));
                functionids.add(number);
            }
        }
        if (functionids.size() > 0) {
            List<Functions> functions = functionsMapper.selectList(new QueryWrapper<Functions>().in("functionid", functionids));

            ArrayList<Long> itemids = new ArrayList<>();
            for (Functions f : functions) {
                itemids.add(f.getItemid());
            }
            List<ResItemList> resItemLists = itemsMapper.itemList(new QueryWrapper<Items>().in("i.itemid", itemids));
            ArrayList<Long> hostids = new ArrayList<>();
            for (ResItemList r : resItemLists) {
                hostids.add(r.getHostid());
            }
            List<Hosts> hosts = hostsMapper.selectList(new QueryWrapper<Hosts>().select("host,hostid").in("hostid", hostids));
            if (resItemLists.size() > 0) {
                for (Triggers r : result) {
                    String expression = r.getExpression();
                    String expression_html = r.getExpression();
                    Pattern pattern = Pattern.compile("\\{(\\d+)\\}");
                    Matcher matcher = pattern.matcher(expression);
                    while (matcher.find()) {
                        String str = matcher.group();
                        for (Functions f : functions) {
                            String parameter = f.getParameter();
                            String parameter_html = null;
                            if (f.getFunctionid().equals(Long.valueOf(str.replace("{", "").replace("}", "")))) {
                                for (ResItemList rr : resItemLists) {
                                    if (rr.getItemid().equals(f.getItemid())) {
                                        for (Hosts h : hosts) {
                                            if (h.getHostid().equals(rr.getHostid())) {
                                                parameter = parameter.replace("$", "/" + h.getHost() + "/" + rr.getKey_());
                                                parameter_html = "<a " + "class='link'" + "data-itemid='" + rr.getItemid() + "'>" + parameter + "</a>";
                                                expression_html = expression_html.replace(str, f.getName() + "(" + parameter_html + ")");
                                                expression = expression.replace(str, f.getName() + "(" + parameter + ")");
                                                break;
                                            }
                                        }
                                        break;
                                    }
                                }
                                break;
                            }
                        }
                    }
                    r.setExpression(expression);
                    r.setExpression_html(expression_html);
                }
            }
        }


        if (null != args.getSelectTags() && args.getSelectTags()) {
            List<TriggerTag> triggerTags = triggerTagMapper.selectList(new QueryWrapper<TriggerTag>().in("triggerid", triggerids));
            for (Triggers r : result) {
                r.setTags(triggerTags.stream()
                        .filter(i -> i.getTriggerid().equals(r.getTriggerid()))
                        .collect(Collectors.toList()));
            }
        }

        return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT, result, count, page.getNum(), page.getSize()));

    }

    @Data
    public static class ArgDetail {
        Long triggerid;
    }

    @Override
    public Result<JSONObject> detail(ArgDetail args) {
        Long triggerid = args.triggerid;
        QueryWrapper<Triggers> qw = new QueryWrapper<>();
        qw.eq("triggerid", triggerid);
        Integer count = triggersMapper.selectCount(qw);
        if (!count.equals(1)) {
            return Result.fail(HttpStatusEnum.BAD_REQUEST, "无效id");
        }
        Triggers trigger = triggersMapper.selectOne(qw);

        //  处理expression部分的macro
        ArrayList<Integer> functionids = new ArrayList<>();
        Pattern pattern = Pattern.compile("\\{(\\d+)\\}");
        Matcher matcher = pattern.matcher(trigger.getExpression());
        while (matcher.find()) {
            Integer number = Integer.valueOf(matcher.group().replace("{", "").replace("}", ""));
            functionids.add(number);
        }

        if (functionids.size() > 0) {
            List<Functions> functions = functionsMapper.selectList(new QueryWrapper<Functions>().in("functionid", functionids));

            ArrayList<Long> itemids = new ArrayList<>();
            for (Functions f : functions) {
                itemids.add(f.getItemid());
            }
            List<ResItemList> resItemLists = itemsMapper.itemList(new QueryWrapper<Items>().in("i.itemid", itemids));
            ArrayList<Long> hostids = new ArrayList<>();
            for (ResItemList r : resItemLists) {
                hostids.add(r.getHostid());
            }
            List<Hosts> hosts = hostsMapper.selectList(new QueryWrapper<Hosts>().select("host,hostid").in("hostid", hostids));
            if (resItemLists.size() > 0) {
                String expression = trigger.getExpression();
                 pattern = Pattern.compile("\\{(\\d+)\\}");
                 matcher = pattern.matcher(trigger.getExpression());
                while (matcher.find()) {
                    String str = matcher.group();
                    for (Functions f : functions) {
                        String parameter = f.getParameter();
                        if (f.getFunctionid().equals(Long.valueOf(str.replace("{", "").replace("}", "")))) {
                            for (ResItemList rr : resItemLists) {
                                if (rr.getItemid().equals(f.getItemid())) {
                                    for (Hosts h : hosts) {
                                        if (h.getHostid().equals(rr.getHostid())) {
                                            parameter = parameter.replace("$", "/" + h.getHost() + "/" + rr.getKey_());
                                            expression = expression.replace(str, f.getName() + "(" + parameter + ")");
                                            break;
                                        }
                                    }
                                    break;
                                }
                            }
                            break;
                        }
                    }
                }
                trigger.setExpression(expression);
            }
        }

        List<TriggerTag> triggerTags = triggerTagMapper.selectList(new QueryWrapper<TriggerTag>().eq("triggerid", triggerid));
        trigger.setTags(triggerTags);

        return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT, trigger));
    }

    @Data
    public static class ArgCreate {
        @NotNull(message = "描述不能为空")
        String description;
        Integer priority;
        @NotNull(message = "表达式不能为空")
        String expression;
    }

    @Override
    public Result<JSONObject> create(TriggersServiceImpl.ArgCreate args) {
        JSONObject params = new JSONObject();
        params.put("description", args.description);
        params.put("priority", args.priority);
        params.put("expression", args.expression);

        return Result.success(ZabbixApi.post("trigger.create", Config.AUTH, params));
    }

    @Data
    public static class ArgDelete {
        ArrayList<Long> triggerids;
    }

    @Override
    public Result<JSONObject> delete(ArgDelete args) {
        return Result.success(ZabbixApi.post("trigger.delete", Config.AUTH, args.triggerids));
    }

    @Data
    public static class ArgUpdate {
        @NotNull(message = "触发器id不能为空")
        Long triggerid;
        String description;
        Integer priority;
        String expression;
    }

    @Override
    public Result<JSONObject> update(ArgUpdate args) {
        JSONObject params = new JSONObject();
        params.put("triggerid", args.triggerid);
        params.put("description", args.description);
        params.put("priority", args.priority);
        params.put("expression", args.expression);

        return Result.success(ZabbixApi.post("trigger.update", Config.AUTH, params));
    }


    @Data
    public static class ArgSwitchStatus {
        ArrayList<Long> triggerids;
        @NotNull(message = "状态不能为空")
        @Min(value = 0, message = "0为开启;1为关闭")
        @Max(value = 1, message = "0为开启;1为关闭")
        Integer status;
    }

    @Override
    public Result<JSONObject> switchStatus(ArgSwitchStatus args) {
        UpdateWrapper<Triggers> uw = new UpdateWrapper<>();
        uw.set("status", args.status);
        uw.in("triggerid", args.triggerids);
        triggersMapper.update(null, uw);
        return Result.success(null);
    }


    @Data
    public static class ArgGetTriggerprototype {
        Integer pageNum;
        Integer pageSize;
        ArrayList<Long> discoveryids;
        Boolean selectTags;
    }

    @Override
    public Result<JSONObject> getTriggerprototype(ArgGetTriggerprototype args) {
        QueryWrapper<Triggers> qw = new QueryWrapper<>();
        qw.eq("t.flags", Config.host.ZBX_FLAG_DISCOVERY_PROTOTYPE);

        ArrayList<Long> arg_itemids = args.discoveryids;
        if (null != arg_itemids && arg_itemids.size() > 0) {
            qw.in("id.parent_itemid", arg_itemids);
        }

        Integer count = triggersMapper.countTriggerprototype(qw);

        ArrayList<Long> triggerids = new ArrayList<>();
        ServiceUtil.Page page = ServiceUtil.init_page(args.pageNum, args.pageSize,count);
        qw.orderByAsc("UPPER(t.description)");
        qw.last("LIMIT " + (page.getNum() - 1) * page.getSize() + ", " + page.getSize());
        List<Triggers> triggeridsList = triggersMapper.triggerprototypeids(qw);
        for (Triggers t : triggeridsList) {
            triggerids.add(t.getTriggerid());
        }
        if (triggerids.size() == 0) {
            return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT, new ArrayList<>(),count));
        }

        List<Triggers> result = triggersMapper.selectList(new QueryWrapper<Triggers>().in("triggerid", triggerids).orderByAsc("UPPER(description)"));

        //  处理expression部分的macro
        ArrayList<Integer> functionids = new ArrayList<>();
        for (Triggers r : result) {
            Pattern pattern = Pattern.compile("\\{(\\d+)\\}");
            Matcher matcher = pattern.matcher(r.getExpression());
            while (matcher.find()) {
                Integer number = Integer.valueOf(matcher.group().replace("{", "").replace("}", ""));
                functionids.add(number);
            }
        }
        if (functionids.size() > 0) {
            List<Functions> functions = functionsMapper.selectList(new QueryWrapper<Functions>().in("functionid", functionids));

            ArrayList<Long> itemids = new ArrayList<>();
            for (Functions f : functions) {
                itemids.add(f.getItemid());
            }
            List<ResItemList> resItemLists = itemsMapper.itemList(new QueryWrapper<Items>().in("i.itemid", itemids));
            ArrayList<Long> hostids = new ArrayList<>();
            for (ResItemList r : resItemLists) {
                hostids.add(r.getHostid());
            }
            List<Hosts> hosts = hostsMapper.selectList(new QueryWrapper<Hosts>().select("host,hostid").in("hostid", hostids));
            if (resItemLists.size() > 0) {
                for (Triggers r : result) {
                    String expression = r.getExpression();
                    String expression_html = r.getExpression();
                    Pattern pattern = Pattern.compile("\\{(\\d+)\\}");
                    Matcher matcher = pattern.matcher(expression);
                    while (matcher.find()) {
                        String str = matcher.group();
                        for (Functions f : functions) {
                            String parameter = f.getParameter();
                            String parameter_html = null;
                            if (f.getFunctionid().equals(Long.valueOf(str.replace("{", "").replace("}", "")))) {
                                for (ResItemList rr : resItemLists) {
                                    if (rr.getItemid().equals(f.getItemid())) {
                                        for (Hosts h : hosts) {
                                            if (h.getHostid().equals(rr.getHostid())) {
                                                parameter = parameter.replace("$", "/" + h.getHost() + "/" + rr.getKey_());
                                                parameter_html = "<a " + "class='link'" + "data-itemid='" + rr.getItemid() + "'>" + parameter + "</a>";
                                                expression_html = expression_html.replace(str, f.getName() + "(" + parameter_html + ")");
                                                expression = expression.replace(str, f.getName() + "(" + parameter + ")");
                                                break;
                                            }
                                        }
                                        break;
                                    }
                                }
                                break;
                            }
                        }
                    }
                    r.setExpression(expression);
                    r.setExpression_html(expression_html);
                }
            }
        }


        if (null != args.getSelectTags() && args.getSelectTags()) {
            List<TriggerTag> triggerTags = triggerTagMapper.selectList(new QueryWrapper<TriggerTag>().in("triggerid", triggerids));
            for (Triggers r : result) {
                r.setTags(triggerTags.stream()
                        .filter(i -> i.getTriggerid().equals(r.getTriggerid()))
                        .collect(Collectors.toList()));
            }
        }

        return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT, result, count, page.getNum(), page.getSize()));

    }

    @Data
    public static class ArgTriggerprototypeDetail {
        Long triggerid;
    }

    @Override
    public Result<JSONObject> triggerprototypeDetail(ArgTriggerprototypeDetail args) {
        Long triggerid = args.triggerid;
        QueryWrapper<Triggers> qw = new QueryWrapper<>();
        qw.eq("triggerid", triggerid);
        qw.eq("flags", Config.host.ZBX_FLAG_DISCOVERY_PROTOTYPE);
        Integer count = triggersMapper.selectCount(qw);
        if (!count.equals(1)) {
            return Result.fail(HttpStatusEnum.BAD_REQUEST, "无效id");
        }
        Triggers trigger = triggersMapper.selectOne(qw);

        //  处理expression部分的macro
        ArrayList<Integer> functionids = new ArrayList<>();
        Pattern pattern = Pattern.compile("\\{(\\d+)\\}");
        Matcher matcher = pattern.matcher(trigger.getExpression());
        while (matcher.find()) {
            Integer number = Integer.valueOf(matcher.group().replace("{", "").replace("}", ""));
            functionids.add(number);
        }

        if (functionids.size() > 0) {
            List<Functions> functions = functionsMapper.selectList(new QueryWrapper<Functions>().in("functionid", functionids));

            ArrayList<Long> itemids = new ArrayList<>();
            for (Functions f : functions) {
                itemids.add(f.getItemid());
            }
            List<ResItemList> resItemLists = itemsMapper.itemList(new QueryWrapper<Items>().in("i.itemid", itemids));
            ArrayList<Long> hostids = new ArrayList<>();
            for (ResItemList r : resItemLists) {
                hostids.add(r.getHostid());
            }
            List<Hosts> hosts = hostsMapper.selectList(new QueryWrapper<Hosts>().select("host,hostid").in("hostid", hostids));
            if (resItemLists.size() > 0) {
                String expression = trigger.getExpression();
                pattern = Pattern.compile("\\{(\\d+)\\}");
                matcher = pattern.matcher(trigger.getExpression());
                while (matcher.find()) {
                    String str = matcher.group();
                    for (Functions f : functions) {
                        String parameter = f.getParameter();
                        if (f.getFunctionid().equals(Long.valueOf(str.replace("{", "").replace("}", "")))) {
                            for (ResItemList rr : resItemLists) {
                                if (rr.getItemid().equals(f.getItemid())) {
                                    for (Hosts h : hosts) {
                                        if (h.getHostid().equals(rr.getHostid())) {
                                            parameter = parameter.replace("$", "/" + h.getHost() + "/" + rr.getKey_());
                                            expression = expression.replace(str, f.getName() + "(" + parameter + ")");
                                            break;
                                        }
                                    }
                                    break;
                                }
                            }
                            break;
                        }
                    }
                }
                trigger.setExpression(expression);
            }
        }

        List<TriggerTag> triggerTags = triggerTagMapper.selectList(new QueryWrapper<TriggerTag>().eq("triggerid", triggerid));
        trigger.setTags(triggerTags);

        return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT, trigger));
    }

    @Data
    public static class ArgTriggerprototypeCreate {
        @NotNull(message = "描述不能为空")
        String description;
        Integer priority;
        @NotNull(message = "表达式不能为空")
        String expression;
    }

    @Override
    public Result<JSONObject> triggerprototypeCreate(ArgTriggerprototypeCreate args) {
        JSONObject params = new JSONObject();
        params.put("description", args.description);
        params.put("priority", args.priority);
        params.put("expression", args.expression);

        return Result.success(ZabbixApi.post("triggerprototype.create", Config.AUTH, params));
    }

    @Data
    public static class ArgTriggerprototypeDelete {
        ArrayList<Long> triggerids;
    }

    @Override
    public Result<JSONObject> triggerprototypeDelete(ArgTriggerprototypeDelete args) {
        return Result.success(ZabbixApi.post("triggerprototype.delete", Config.AUTH, args.triggerids));
    }

    @Data
    public static class ArgTriggerprototypeUpdate {
        @NotNull(message = "触发器id不能为空")
        Long triggerid;
        String description;
        Integer priority;
        String expression;
    }

    @Override
    public Result<JSONObject> triggerprototypeUpdate(ArgTriggerprototypeUpdate args) {
        JSONObject params = new JSONObject();
        params.put("triggerid", args.triggerid);
        params.put("description", args.description);
        params.put("priority", args.priority);
        params.put("expression", args.expression);

        return Result.success(ZabbixApi.post("triggerprototype.update", Config.AUTH, params));
    }


    @Data
    public static class ArgTriggerprototypeSwitchStatus {
        ArrayList<Long> triggerids;
        @NotNull(message = "状态不能为空")
        @Min(value = 0, message = "0为开启;1为关闭")
        @Max(value = 1, message = "0为开启;1为关闭")
        Integer status;
    }

    @Override
    public Result<JSONObject> triggerprototypeSwitchStatus(ArgTriggerprototypeSwitchStatus args) {
        UpdateWrapper<Triggers> uw = new UpdateWrapper<>();
        uw.set("status", args.status);
        uw.in("triggerid", args.triggerids);
        triggersMapper.update(null, uw);
        return Result.success(null);
    }
}
