package com.zh.test_zabbix.serviceImpl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zh.test_zabbix.config.Config;
import com.zh.test_zabbix.mapper.*;
import com.zh.test_zabbix.mapper.res.ResUsers;
import com.zh.test_zabbix.model.MediaType;
import com.zh.test_zabbix.model.Role;
import com.zh.test_zabbix.model.Sessions;
import com.zh.test_zabbix.model.Users;
import com.zh.test_zabbix.service.IUsersService;
import com.zh.test_zabbix.serviceImpl.util.ServiceUtil;
import com.zh.test_zabbix.utils.Common;
import com.zh.test_zabbix.utils.Valid;
import com.zh.test_zabbix.utils.result.Result;
import lombok.Data;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author zh
 * @since 2023-11-22
 */
@Service
public class UsersServiceImpl extends ServiceImpl<UsersMapper, Users> implements IUsersService {

    @Resource
    private UsersMapper usersMapper;

    @Resource
    private UsrgrpMapper usrgrpMapper;

    @Resource
    private RoleMapper roleMapper;

    @Resource
    private SessionsMapper sessionsMapper;

    @Resource
    private MediaTypeMapper  mediaTypeMapper;

    @Data
    public static class ArgGet {
        Integer pageNum;
        Integer pageSize;
        ArrayList<Long> roleids;
        ArrayList<Long> usrgrpids;
        String username;
        String name;
        String surname;
        Boolean selectGroups;
        Boolean selectRole;
        Boolean selectLastAccess;
    }

    @Override
    public Result<JSONObject> get(ArgGet args) {
        // 是否page格式
        Boolean isPage = true;
        if (null != args.pageSize && args.pageSize == 0) {
            isPage = false;
        }

        QueryWrapper<Users> qw = new QueryWrapper<>();
        qw.in(Valid.isNotNull(args.roleids), "roleid", args.roleids);
        qw.inSql(Valid.isNotNull(args.usrgrpids), "userid", "SELECT userid FROM users_groups WHERE usrgrpid in (" + Common.getArrayList(args.usrgrpids) + ")");
        qw.like(Valid.isNotNull(args.username), "UPPER(username)", (null == args.username ? "" : args.username).toUpperCase());
        qw.like(Valid.isNotNull(args.name), "UPPER(name)", (null == args.name ? "" : args.name).toUpperCase());
        qw.like(Valid.isNotNull(args.surname), "UPPER(surname)", (null == args.surname ? "" : args.surname).toUpperCase());

        Integer count = usersMapper.selectCount(qw);
        if (count == 0) {
            return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT, new ArrayList<>(), count));
        }

        ServiceUtil.Page page = new ServiceUtil.Page();
        if (isPage) {
            page = ServiceUtil.init_page(args.pageNum, args.pageSize,count);
            qw.last("LIMIT " + (page.getNum() - 1) * page.getSize() + ", " + page.getSize());
        }
        qw.orderByAsc("UPPER(username)");

        List<ResUsers> result = usersMapper.get(qw);

        ArrayList<Long> userids = new ArrayList<>();

        for (ResUsers user : result) {
            userids.add(user.getUserid());
        }

        List<Map> user_status = usersMapper.user_status(new QueryWrapper<Users>().in("ug.userid", userids));
        for (ResUsers r : result) {
            for (Map us : user_status) {
                if (Long.valueOf(us.get("userid").toString()).equals(r.getUserid())) {
                    r.setUsers_status(Integer.parseInt(us.get("users_status").toString()));
                    break;
                }
            }
        }

        if (Valid.isTrue(args.selectGroups)) {
            List<Map> usrgrps = usrgrpMapper.getByUsers(new QueryWrapper<Users>().in("t.userid", userids));
            for (ResUsers r : result) {
                List<Map> collect = usrgrps.stream()
                        .filter(i -> Long.valueOf(i.get("userid").toString()).equals(r.getUserid()))
                        .collect(Collectors.toList());
                r.setUsergroups(collect);
            }
        }

        if (Valid.isTrue(args.selectRole)) {
            List<Map> roles = roleMapper.get(new QueryWrapper<Role>().in("u.userid", userids));
            for (ResUsers r : result) {
                for (Map i : roles) {
                    if (Long.valueOf(i.get("userid").toString()).equals(r.getUserid())) {
                        r.setRole(i.get("name").toString());
                        break;
                    }
                }
            }
        }

        if(Valid.isTrue(args.selectLastAccess)){
            List<Sessions> sessions = sessionsMapper.selectList(new QueryWrapper<Sessions>()
                    .select("userid,MAX(lastaccess) AS lastaccess")
                    .in("userid", userids)
                    .eq("status", Config.User.ZBX_SESSION_ACTIVE)
                    .groupBy("userid"));
            for (ResUsers r : result) {
                for (Sessions i : sessions) {
                    if (i.getUserid().equals(r.getUserid())) {
                        r.setLastaccess(i.getLastaccess());
                        break;
                    }
                }
            }
        }

        if (isPage) {
            return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT, result, count, page.getNum(), page.getSize()));
        } else {
            return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT, result));
        }
    }


    @Override
    public Result<JSONObject> medias() {
        List<MediaType> result = mediaTypeMapper.selectList(new QueryWrapper<MediaType>().eq("status", Config.User.MEDIA_STATUS_ACTIVE));
        return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT, result));
    }
}
