package com.zh.test_zabbix.serviceImpl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zh.test_zabbix.config.Config;
import com.zh.test_zabbix.mapper.UsersGroupsMapper;
import com.zh.test_zabbix.mapper.UsersMapper;
import com.zh.test_zabbix.mapper.UsrgrpMapper;
import com.zh.test_zabbix.model.Users;
import com.zh.test_zabbix.model.Usrgrp;
import com.zh.test_zabbix.service.IUsrgrpService;
import com.zh.test_zabbix.serviceImpl.util.ServiceUtil;
import com.zh.test_zabbix.utils.Valid;
import com.zh.test_zabbix.utils.ZabbixApi;
import com.zh.test_zabbix.utils.result.HttpStatusEnum;
import com.zh.test_zabbix.utils.result.Result;
import lombok.Data;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.validation.constraints.NotNull;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author zh
 * @since 2023-11-22
 */
@Service
public class UsrgrpServiceImpl extends ServiceImpl<UsrgrpMapper, Usrgrp> implements IUsrgrpService {

    @Resource
    private UsersMapper usersMapper;

    @Resource
    private UsrgrpMapper usrgrpMapper;

    @Resource
    private UsersGroupsMapper usersGroupsMapper;

    @Data
    public static class ArgGet {
        Integer pageNum;
        Integer pageSize;
        String name;
        Integer status; // 0 启用 1 停用

        Boolean selectUsers;
    }

    @Override
    public Result<JSONObject> get(ArgGet args) {
        // 是否page格式
        Boolean isPage = true;
        if (null != args.pageSize && args.pageSize == 0) {
            isPage = false;
        }

        QueryWrapper<Usrgrp> qw = new QueryWrapper<>();
        qw.like(Valid.isNotNull(args.name),"UPPER(name)", (null == args.name ? "" : args.name).toUpperCase());
        qw.eq(Valid.isNotNull(args.status),"users_status",args.status);

        Integer count = usrgrpMapper.selectCount(qw);
        if (count == 0) {
            return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT, new ArrayList<>(), count));
        }

        ServiceUtil.Page page = new ServiceUtil.Page();
        if (isPage) {
            page = ServiceUtil.init_page(args.pageNum, args.pageSize,count);
            qw.last("LIMIT " + (page.getNum() - 1) * page.getSize() + ", " + page.getSize());
        }
        qw.orderByAsc("UPPER(name)");

        List<Usrgrp> result = usrgrpMapper.selectList(qw);

        ArrayList<Long> usrgrpids = new ArrayList<>();
        for (Usrgrp ug : result) {
            usrgrpids.add(ug.getUsrgrpid());
        }

        if(Valid.isTrue(args.selectUsers)) {
            List<Map> users = usersMapper.getByUsrGrp(new QueryWrapper<Usrgrp>().in("t.usrgrpid", usrgrpids));
            HashSet<Long> userids = new HashSet<>();
            for (Map u : users) {
                userids.add(Long.valueOf(u.get("userid").toString()));
            }
            List<Map> user_status = usersMapper.user_status(new QueryWrapper<Users>().in("ug.userid", userids));
            for (Map u : users) {
                for (Map us : user_status) {
                    if (Long.valueOf(us.get("userid").toString()).equals(Long.valueOf(u.get("userid").toString()))) {
                        u.put("users_status",Integer.parseInt(us.get("users_status").toString()));
                        break;
                    }
                }
            }
            for (Usrgrp r : result) {
                List<Map> collect = users.stream()
                        .filter(i -> Long.valueOf(i.get("usrgrpid").toString()).equals(r.getUsrgrpid()))
                        .collect(Collectors.toList());
                r.setUsers(collect);
            }
        }

        if (isPage) {
            return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT, result, count, page.getNum(), page.getSize()));
        } else {
            return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT, result));
        }
    }

    @Data
    public static class ArgDetail {
        @NotNull
        Long usrgrpid;
    }

    @Override
    public Result<JSONObject> detail(ArgDetail args) {
        Long usrgrpid = args.usrgrpid;
        QueryWrapper<Usrgrp> qw = new QueryWrapper<>();
        qw.eq("usrgrpid", usrgrpid);
        Integer count = usrgrpMapper.selectCount(qw);
        if (!count.equals(1)) {
            return Result.fail(HttpStatusEnum.BAD_REQUEST, "无效id");
        }

        Usrgrp result = usrgrpMapper.selectOne(qw);

        List<Map> users = usersMapper.getByUsrGrp(new QueryWrapper<Usrgrp>().eq("t.usrgrpid", usrgrpid));
        result.setUsers(users);

        List<Map> rights = usrgrpMapper.rights(new QueryWrapper<Usrgrp>().eq("r.groupid",usrgrpid));
        result.setRights(rights);

        return Result.success(ServiceUtil.ResStruct(Config.MS_RESULT, result));
    }

    @Data
    public static class ArgCreate {
        String name;
        List<ArgRights> hostgroup_rights;
        List<ArgRights> templategroup_rights;
       List<ArgUsers> users;
    }

    @Data
    public static class ArgRights{
        Integer permission;
        Long id;
    }

    @Data
    public static class ArgUsers{
        Long userid;
    }

    @Override
    public Result<JSONObject> create(ArgCreate args) {
        JSONObject params = new JSONObject();
        params.put("name", args.name);
        params.put("hostgroup_rights", args.hostgroup_rights);
        params.put("templategroup_rights", args.templategroup_rights);
        params.put("users", args.users);

        JSONObject res = ZabbixApi.post( "usergroup.create", Config.AUTH, params);
        return Result.success(res);
    }

    @Data
    public static class ArgDelete {
        ArrayList<Long> usrgrpids;
    }

    @Override
    public Result<JSONObject> delete(ArgDelete args) {
        JSONObject res = ZabbixApi.post( "usergroup.delete", Config.AUTH, args.usrgrpids);
        return Result.success(res);
    }

    @Data
    public static class ArgUpdate {
        @NotNull(message = "用户组不能为空")
        Long usrgrpid;
        Integer users_status;
        String name;
        List<ArgRights> hostgroup_rights;
        List<ArgRights> templategroup_rights;
        List<ArgUsers> users;
    }

    @Override
    public Result<JSONObject> update(ArgUpdate args) {
        JSONObject params = new JSONObject();
        params.put("users_status", args.users_status);
        params.put("usrgrpid", args.usrgrpid);
        params.put("name", args.name);
        params.put("hostgroup_rights", args.hostgroup_rights);
        params.put("templategroup_rights", args.templategroup_rights);
        params.put("users", args.users);

        JSONObject res = ZabbixApi.post("usergroup.update", Config.AUTH, params);
        return Result.success(res);
    }

}
